@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # <xbar.title>Meta Package Manager</xbar.title>
-# <xbar.version>v5.3.0</xbar.version>
+# <xbar.version>v5.6.0</xbar.version>
 # <xbar.author>Kevin Deldycke</xbar.author>
 # <xbar.author.github>kdeldycke</xbar.author.github>
 # <xbar.desc>List outdated packages and manage upgrades.</xbar.desc>
@@ -9,7 +9,7 @@
 # <xbar.abouturl>https://github.com/kdeldycke/meta-package-manager</xbar.abouturl>
 # <xbar.var>boolean(VAR_SUBMENU_LAYOUT=false): Group packages into a sub-menu for each manager.</xbar.var>
 # <xbar.var>boolean(VAR_TABLE_RENDERING=true): Aligns package names and versions in a table for easier visual parsing.</xbar.var>
-# XXX Deactive font-related options for Xbar. Default variable value does not allow `=` character in Xbar. See: https://github.com/matryer/xbar/issues/832
+# XXX Deactivate font-related options for Xbar. Default variable value does not allow `=` character in Xbar. See: https://github.com/matryer/xbar/issues/832
 # <!--xbar.var>string(VAR_DEFAULT_FONT=""): Default font to use for non-monospaced text.</xbar.var-->
 # <!--xbar.var>string(VAR_MONOSPACE_FONT="font=Menlo size=12"): Default configuration for monospace fonts, including errors. Is used for table rendering.</xbar.var-->
 # <swiftbar.environment>[VAR_SUBMENU_LAYOUT: false, VAR_TABLE_RENDERING: true, VAR_DEFAULT_FONT: , VAR_MONOSPACE_FONT: font=Menlo size=12]</swiftbar.environment>
@@ -20,10 +20,6 @@ Default update cycle is set to 7 hours so we have a chance to get user's
 attention once a day. Higher frequency might ruin the system as all checks are
 quite resource intensive, and Homebrew might hit GitHub's API calls quota.
 
-Minimal requirement is macOS Catalina (10.15) for both Xbar and SwiftBar.
-Catalina deprecates Python 2.x, and ships with Python 3.7.3. So this plugin is
-required to work with Python 3.7.3 or newer.
-
 - Xbar automatically bridge plugin options between its UI and environment
   variable on script execution. See:
   https://xbarapp.com/docs/2021/03/14/variables-in-xbar.html
@@ -31,10 +27,30 @@ required to work with Python 3.7.3 or newer.
 - This is in progress for SwiftBar at:
   https://github.com/swiftbar/SwiftBar/issues/160
 """
+import sys
 
+python_min_version = (3, 7, 3)
+"""Minimal requirement is macOS Catalina (10.15) for both Xbar and SwiftBar.
+
+Catalina deprecates Python 2.x, and ships with Python 3.7.3. So this plugin is
+required to work with Python 3.7.3 or newer.
+"""
+
+
+def v_to_str(version_tuple):
+    """Transforms into a string a tuple of integers representing a version."""
+    return ".".join(map(str, version_tuple))
+
+
+if sys.version_info < python_min_version:
+    raise SystemError(
+        f"Bar plugin ran with Python {sys.version}, "
+        f"but requires Python >= {v_to_str(python_min_version)}"
+    )
+
+import argparse
 import os
 import re
-import sys
 from configparser import RawConfigParser
 from shutil import which
 from subprocess import run
@@ -63,11 +79,11 @@ class MPMPlugin:
     @staticmethod
     def extended_environment():
         """Returns a tweaked environment extending global path to find non-default
-        system- wide binaries.
+        system-wide binaries.
 
         macOS does not put ``/usr/local/bin`` or ``/opt/local/bin`` in the ``PATH`` for
-        GUI apps. For some package managers this is a problem. Additioanlly Homebrew and
-        Macports are using different pathes. So, to make sure we can always get to the
+        GUI apps. For some package managers this is a problem. Additionally Homebrew and
+        Macports are using different paths. So, to make sure we can always get to the
         necessary binaries, we overload the path. Current preference order would equate
         to Homebrew, Macports, then system.
         """
@@ -180,10 +196,41 @@ class MPMPlugin:
     def mpm_exec(self):
         """Search for mpm execution alternatives, either direct ``mpm`` call or as an
         executable Python module."""
-        mpm_exec = (self.locate_bin("mpm"),)
-        if not mpm_exec:
-            mpm_exec = (self.python_path, "-m", "meta_package_manager")
-        return mpm_exec
+        # XXX Local debugging and development.
+        # return "poetry", "run", "mpm"
+        mpm_exec = self.locate_bin("mpm")
+        if mpm_exec:
+            return (mpm_exec,)
+        return self.python_path, "-m", "meta_package_manager"
+
+    def check_mpm(self):
+        """Test-run mpm execution and extract its version."""
+        error = None
+        try:
+            process = run(
+                (*self.mpm_exec, "--version"), capture_output=True, encoding="utf-8"
+            )
+            error = process.stderr
+        except FileNotFoundError as ex:
+            error = ex
+
+        installed = False
+        mpm_version = None
+        up_to_date = False
+        # Is mpm CLI installed on the system?
+        if not process.returncode and not error:
+            installed = True
+            # Is mpm too old?
+            version_string = (
+                re.compile(r".*\s+(?P<version>[0-9\.]+)$", re.MULTILINE)
+                .search(process.stdout)
+                .groupdict()["version"]
+            )
+            mpm_version = tuple(map(int, version_string.split(".")))
+            if mpm_version >= self.mpm_min_version:
+                up_to_date = True
+
+        return installed, mpm_version, up_to_date, error
 
     @staticmethod
     def pp(*args):
@@ -224,37 +271,13 @@ class MPMPlugin:
 
     def print_menu(self):
         """Print the main menu."""
-        # Test mpm execution.
-        error = None
-        try:
-            process = run(
-                (*self.mpm_exec, "--version"), capture_output=True, encoding="utf-8"
-            )
-            error = process.stderr
-        except FileNotFoundError as ex:
-            error = ex
-
-        mpm_installed = False
-        mpm_up_to_date = False
-        # Is mpm CLI installed on the system?
-        if not process.returncode and not error:
-            mpm_installed = True
-            # Is mpm too old?
-            version_string = (
-                re.compile(r".*\s+(?P<version>[0-9\.]+)$", re.MULTILINE)
-                .search(process.stdout)
-                .groupdict()["version"]
-            )
-            mpm_version = tuple(map(int, version_string.split(".")))
-            if mpm_version >= self.mpm_min_version:
-                mpm_up_to_date = True
-
+        mpm_installed, _, mpm_up_to_date, error = self.check_mpm()
         if not mpm_installed or not mpm_up_to_date:
             self.print_error_header()
             self.print_error(error)
             print("---")
             action_msg = "Install" if not mpm_installed else "Upgrade"
-            min_version_str = ".".join(map(str, self.mpm_min_version))
+            min_version_str = v_to_str(self.mpm_min_version)
             self.pp(
                 f"{action_msg} mpm >= v{min_version_str}",
                 f"shell={self.python_path}",
@@ -279,7 +302,10 @@ class MPMPlugin:
         # Fetch outdated packages from all package managers available on the system.
         # We defer all rendering to mpm itself so it can compute more intricate layouts.
         process = run(
-            (*self.mpm_exec, "--verbosity", "ERROR", "outdated", "--plugin-output"),
+            # We silence all errors but the CRITICAL ones. All others will be captured
+            # by mpm in --plugin-output mode and rendered back into each manager
+            # section.
+            (*self.mpm_exec, "--verbosity", "CRITICAL", "outdated", "--plugin-output"),
             capture_output=True,
             encoding="utf-8",
         )
@@ -297,7 +323,29 @@ class MPMPlugin:
 
 if __name__ == "__main__":
 
+    parser = argparse.ArgumentParser()
+    parser.add_argument(
+        "--check-mpm",
+        action="store_true",
+        help="Locate mpm on the system and check its version.",
+    )
+    args = parser.parse_args()
+
     # Wrap plugin execution with our custom environment variables to avoid
     # environment leaks.
     with patch.dict("os.environ", MPMPlugin.extended_environment()):
-        MPMPlugin().print_menu()
+        plugin = MPMPlugin()
+
+        if args.check_mpm:
+            mpm_installed, mpm_version, mpm_up_to_date, error = plugin.check_mpm()
+            if not mpm_installed:
+                raise FileNotFoundError(error)
+            if not mpm_up_to_date:
+                raise ValueError(
+                    f"{plugin.mpm_exec} is too old: "
+                    f"{v_to_str(mpm_version)} < {v_to_str(plugin.mpm_min_version)}"
+                )
+            print(f"{' '.join(plugin.mpm_exec)} v{v_to_str(mpm_version)}")
+
+        else:
+            plugin.print_menu()
