@@ -15,14 +15,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import os
 import re
 import sys
 from contextlib import nullcontext
-from dataclasses import dataclass
+from dataclasses import InitVar, asdict, dataclass
+from enum import Enum
 from pathlib import Path
 from shutil import which
 from textwrap import dedent, indent, shorten
+from typing import ContextManager, Dict, Iterable, Iterator, Optional, Union, cast
 from unittest.mock import patch
 
 if sys.version_info >= (3, 8):
@@ -32,27 +36,46 @@ else:
 
 from boltons.iterutils import flatten
 from boltons.strutils import strip_ansi
-from boltons.typeutils import classproperty
 from click_extra.colorize import theme
 from click_extra.platform import CURRENT_OS_ID, is_linux
 from click_extra.run import INDENT, format_cli, run_cmd
+from click_extra.tests.conftest import ExtraCliRunner
 
 from . import logger
-from .version import parse_version
+from .version import TokenizedString, parse_version
+
+Operations = Enum(
+    "Operations",
+    (
+        "installed",
+        "outdated",
+        "search",
+        "install",
+        "upgrade",
+        "upgrade_all",
+        "remove",
+        "sync",
+        "cleanup",
+    ),
+)
+"""Recognize operation IDs that are implemented by package manager with their specific CLI invocation.
+
+Each operation has its own CLI subcommand.
+"""
 
 
 class CLIError(Exception):
 
     """An error occurred when running package manager CLI."""
 
-    def __init__(self, code, output, error):
+    def __init__(self, code: int, output: str, error: str):
         """The exception internally keeps the result of CLI execution."""
         super().__init__()
         self.code = code
         self.output = output
         self.error = error
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Human-readable error."""
         indented_output = indent(str(self.output), INDENT)
         indented_error = indent(str(self.error), INDENT)
@@ -68,7 +91,7 @@ class CLIError(Exception):
             INDENT,
         )
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         error_excerpt = shorten(
             " ".join(self.error.split()), width=60, placeholder="(...)"
         )
@@ -80,52 +103,106 @@ class Package:
     """Lightweight representation of a package and its metadata."""
 
     id: str
-    name: str = None
-    description: str = None
-    installed_version: str = None
-    latest_version: str = None
+    """ID is required and is the primary key used by the manager. """
+
+    name: str | None = None
+
+    label: str | None = None
+    """Label is used in *Bar plugin to display user-friendly entries."""
+
+    description: str | None = None
+
+    installed_version: TokenizedString | str | None = None
+    latest_version: TokenizedString | str | None = None
+    """ ``installed_version`` and ``latest_version`` are allowed to temporarily be a string
+    between ``__init__`` and ``__post_init__``. Once they reach the later, they're parsed and
+    normalized into ``TokenizedString``.
+    """
+
+    upgrade_cli: tuple[str, ...] | None = None
+
+    manager: InitVar[PackageManager] = None
+    """Init-only variable user to populate package metadata requiring a lookup from the manager."""
+
+    def __post_init__(self, manager) -> None:
+        self.label = self.name if self.name else self.id
 
-    def __post_init__(self):
-        # XXX A name is not required but the table rendering code is not capable
-        # of rendering None values yet.
-        if not self.name:
-            self.name = self.id
         # Make sure version strings are parsed into proper objects.
         self.installed_version = parse_version(self.installed_version)
         self.latest_version = parse_version(self.latest_version)
 
+        try:
+            self.upgrade_cli = manager.upgrade_one_cli(self.id)
+        except NotImplementedError:
+            pass
 
-class PackageManager:
+
+def packages_asdict(packages: Iterator[Package], keep_fields: tuple[str, ...]):
+    """Returns a list of packages casted to a ``dict`` with only a subset of its
+    fields."""
+    return ({k: v for k, v in asdict(p).items() if k in keep_fields} for p in packages)
+
+
+_EnvVars = Optional[Dict[str, str]]
+
+# XXX Recursive types are not supported by mypy yet: https://github.com/python/mypy/issues/731
+# _NestedArgs = Iterable[Union[str, Path, None, Iterable["_NestedArgs"]]]
+_Arg = Union[str, Path, None]
+_Args = Iterable[_Arg]
+_NestedArgs = Iterable[
+    Union[
+        _Arg, Iterable[Union[_Arg, Iterable[Union[_Arg, Iterable[Union[_Arg, _Args]]]]]]
+    ]
+]
+
+
+class MetaPackageManager(type):
+    """Custom metaclass used as a class factory for package managers."""
+
+    def __init__(cls, name, bases, dct):
+        """Sets some class defaults, but only if they're not redefined in the final
+        manager class."""
+
+        if "id" not in dct:
+            cls.id = name.lower().replace("_", "-")
+
+        if "name" not in dct:
+            cls.name = name
+
+        if "cli_names" not in dct:
+            cls.cli_names = (cls.id,)
+
+        if "virtual" not in dct:
+            cls.virtual = name == "PackageManager" or not cls.cli_names
+
+
+class PackageManager(metaclass=MetaPackageManager):
 
     """Base class from which all package manager definitions inherits."""
 
-    @classproperty
-    def id(cls):
-        """Returns package manager's ID.
+    id: str
+    """Package manager's ID.
 
-        Derived by defaults from the lower-cased class name in which underscores ``_`` are replaced
-        by dashes ``-``.
+    Derived by defaults from the lower-cased class name in which underscores ``_`` are replaced
+    by dashes ``-``.
 
-        This ID must be unique among all package manager definitions and lower-case, as
-        they're used as feature flags for the :program:`mpm` CLI.
-        """
-        return cls.__name__.lower().replace("_", "-")
+    This ID must be unique among all package manager definitions and lower-case, as
+    they're used as feature flags for the :program:`mpm` CLI.
+    """
 
-    @classproperty
-    def name(cls):
-        """Return package manager's common name.
+    name: str
+    """Return package manager's common name.
 
-        Defaults based on class name.
-        """
-        return cls.__name__
+    Default value is based on class name.
+    """
 
-    homepage_url = None
+    homepage_url: str | None = None
     """Home page of the project, only used in documentation for reference."""
 
-    platforms = frozenset()
+    platforms: frozenset = frozenset()
     """List of platforms supported by the manager."""
 
-    requirement = None
+    requirement: str | None = None
     """Minimal required version.
 
     Should be a string parseable by :py:class:`meta_package_manager.version.parse_version`.
@@ -133,19 +210,24 @@ class PackageManager:
     Defaults to ``None``, which deactivate version check entirely.
     """
 
-    @classproperty
-    def cli_names(cls):
-        """List of CLI names the package manager is known as.
+    cli_names: tuple[str, ...]
+    """List of CLI names the package manager is known as.
 
-        The supported CLI names are ordered by priority. This is used for example to
-        help out the search of the right binary in the case of the python3/python2
-        transition.
+    The supported CLI names are ordered by priority. This is used for example to
+    help out the search of the right binary in the case of the python3/python2
+    transition.
 
-        Is derived by default from the manager's ID.
-        """
-        return (cls.id,)
+    Is derived by default from the manager's ID.
+    """
 
-    cli_search_path = ()
+    virtual: bool
+    """Should we expose the package manager to the user?
+
+    Virtual package manager are just skeleton classes used to factorize code among
+    managers of the same family.
+    """
+
+    cli_search_path: tuple[str, ...] = ()
     """ List of additional path to help :program:`mpm` hunt down the package manager CLI.
 
     Should be a list of strings whose order dictatate the search sequence.
@@ -154,14 +236,14 @@ class PackageManager:
     works well on all platforms.
     """
 
-    extra_env = None
+    extra_env: _EnvVars = None
     """Additional environment variables to add to the current context.
 
     Automatically applied on each :py:func:`meta_package_manager.base.PackageManager.run_cli`
     calls.
     """
 
-    pre_cmds = ()
+    pre_cmds: tuple[str, ...] = ()
     """Global list of pre-commands to add before before invoked CLI.
 
     Automatically added to each :py:func:`meta_package_manager.base.PackageManager.run_cli`
@@ -170,8 +252,8 @@ class PackageManager:
     Used to prepend `sudo <https://www.sudo.ws>`_ or other system utilities.
     """
 
-    pre_args = ()
-    post_args = ()
+    pre_args: tuple[str, ...] = ()
+    post_args: tuple[str, ...] = ()
     """Global list of options used before and after the invoked package manager CLI.
 
     Automatically added to each :py:func:`meta_package_manager.base.PackageManager.run_cli`
@@ -180,10 +262,10 @@ class PackageManager:
     Essentially used to force silencing, low verbosity or no-color output.
     """
 
-    version_cli_options = ("--version",)
+    version_cli_options: tuple[str, ...] = ("--version",)
     """List of options to get the version from the package manager CLI."""
 
-    version_regex = r"(?P<version>\S+)"
+    version_regex: str = r"(?P<version>\S+)"
     """ Regular expression used to extract the version number from the result of the CLI
     run with the options above. It doesn't matter if the regex returns unsanitized
     and crappy string. The :py:func:`meta_package_manager.base.PackageManager.version`
@@ -192,72 +274,120 @@ class PackageManager:
     By default match the first part that is space-separated.
     """
 
-    stop_on_error = False
+    stop_on_error: bool = False
     """Tell the manager to either raise or continue on errors."""
 
-    ignore_auto_updates = True
+    ignore_auto_updates: bool = True
     """Some managers can report or ignore packages which have their own auto-update mechanism.
     """
 
-    dry_run = False
+    dry_run: bool = False
     """Do not actually perform any action, just simulate CLI calls."""
 
-    def __init__(self):
-        """Initialize a ``self.cli_errors`` list to accumulate all CLI errors
-        encountered by the package mananger."""
+    cli_errors: list[CLIError]
+    """Accumulate all CLI errors encountered by the package manager."""
+
+    package_class: type[Package] = Package
+    """The dataclass to use to produce Package objects from the manager."""
+
+    def __init__(self) -> None:
+        """Initialize ``cli_errors`` list."""
         self.cli_errors = []
 
-    @classproperty
-    def virtual(cls):
-        """Should we expose the package manager to the user?
+    @classmethod
+    def implements(cls, op: Operations) -> bool:
+        """Inspect manager's implementation to check for proper support of an
+        operation."""
+        logger.debug(f"Does {cls} implements {op}?")
 
-        Virtual package manager are just skeleton classes used to factorize code among
-        managers of the same family.
+        # General case: the operation and the method implementing it shares the same ID.
+        method_deps: tuple[set[str], ...] = ({op.name},)
+
+        # Special case for single-package `upgrade`: we depends on `upgrade_one_cli()`.
+        if op == Operations.upgrade:
+            method_deps = ({"installed", "upgrade_one_cli"},)
+
+        # For `upgrade_all`: we depends on eother `upgrade_all_cli()`, or we can simulate the latter with a combination of
+        # `outdated()` and `upgrade_one_cli()`.
+        elif op == Operations.upgrade_all:
+            method_deps = ({"upgrade_all_cli"}, {"outdated", "upgrade_one_cli"})
+
+        # If none of the classes in the inheritance hierarchy up to the base one implements the operation, then we can be certain
+        # the manager doesn't implement the operation at all.
+        for klass in cls.mro():
+            if klass is PackageManager:
+                return False
+            # Presence of the operation function is not enough to rules out proper implementation, as it can
+            # be a method that raises NotImplemented error anyway. See for instance the upgrade_all_cli in pip.py:
+            # https://github.com/kdeldycke/meta-package-manager/blob/4acc003bd268a59f5a79cf317be6d25a90878f6d/meta_package_manager/managers/pip.py#L271-L279
+            for method_ids in method_deps:
+                all_deps_found = method_ids.issubset(klass.__dict__)
+                if all_deps_found:
+                    return True
+
+        raise NotImplementedError(f"Can't guess {cls} implementation of {op}.")
+
+    def search_cli(self, cli_name: str) -> Path | None:
+        """Search for a CLI on the system.
+
+        Look for the provided ``cli_name`` in this order:
+          * first in paths provided by
+            :py:attr:`cli_search_path <meta_package_manager.base.PackageManager.cli_search_path>`,
+          * then in all the default places specified by the environment variable (i.e. ``os.getenv("PATH")``).
+
+        Only checks if the file exists. Not its executability.
+
+        Returns ``None`` if the CLI was not found or is not a file.
+
+        ..caution::
+
+            Symlinks are not resolved, because some manager like Homebrew on Linux relies on some
+            sort of symlink-based trickery to set environment variables.
         """
-        return cls.__name__ == "PackageManager" or not cls.cli_names
-
-    @cached_property
-    def cli_path(self):
-        """Fully qualified path to the package manager CLI.
-
-        Automatically search the location of the CLI in the system. Try multiple CLI
-        names within several system path.
-
-        Only checks if the file exists. Its executability will be assessed later. See
-        the :py:func:`meta_package_manager.base.PackageManager.executable` method below.
-
-        Returns ``None`` if no CLI was found or those found were not a file.
-        """
-        # Check if the path exist in any of the environment locations.
+        # Locally extend the environment to add manager-specific search path.
         env_path = ":".join(flatten((self.cli_search_path, os.getenv("PATH"))))
 
-        # Search for multiple CLI names.
-        for name in self.cli_names:
-            cli_path = which(name, mode=os.F_OK, path=env_path)
-            if cli_path:
-                break
-            logger.debug(f"{theme.invoked_command(name)} CLI not found.")
-
-        if not cli_path:
-            return
+        cli_path_found = which(cli_name, mode=os.F_OK, path=env_path)
+        if not cli_path_found:
+            logger.debug(f"{theme.invoked_command(cli_name)} CLI not found.")
+            return None
 
         # Check if path exist and is a file.
-        # Do not resolve symlink here. Some manager like Homebrew on Linux rely on some
-        # sort of synlink trickery to set environment variables.
-        cli_path = Path(cli_path)
+        cli_path = Path(cli_path_found)
         if not cli_path.exists():
-            raise FileNotFoundError(f"{cli_path}")
+            raise FileNotFoundError(cli_path)
         elif not cli_path.is_file():
             logger.warning(f"{theme.invoked_command(cli_path)} is not a file.")
-        else:
-            logger.debug(
-                f"{theme.invoked_command(name)} CLI found at {theme.invoked_command(cli_path)}"
-            )
+            return None
 
+        logger.debug(
+            f"{theme.invoked_command(cli_name)} CLI found at {theme.invoked_command(cli_path)}"
+        )
         return cli_path
 
     @cached_property
-    def version(self):
+    def cli_path(self) -> Path | None:
+        """Fully qualified path to the canonical package manager binary.
+
+        Automatically search the location of the CLI in the system. Try multiple CLI
+        names provided by :py:attr:`cli_names <meta_package_manager.base.PackageManager.cli_names>`,
+        in all system path provided by
+        :py:attr:`cli_search_path <meta_package_manager.base.PackageManager.cli_search_path>`.
+
+        Executability of the CLI will be separatel assessed later by
+        the :py:func:`meta_package_manager.base.PackageManager.executable` method below.
+        """
+        # Search for multiple CLI names.
+        if self.cli_names is not None:
+            for name in self.cli_names:
+                cli_path_found = self.search_cli(name)
+                if cli_path_found:
+                    return cli_path_found
+
+        return None
+
+    @cached_property
+    def version(self) -> TokenizedString | None:
         """Invoke the manager and extract its own reported version string.
 
         Returns a parsed and normalized version in the form of a
@@ -286,7 +416,7 @@ class PackageManager:
                     )
                     # Declare CLI as un-executable.
                     self.executable = False
-                    return
+                    return None
                 # Unidentified error: re-raise.
                 raise
 
@@ -300,14 +430,15 @@ class PackageManager:
                 parsed_version = parse_version(version_string)
                 logger.debug(f"Parsed version: {parsed_version!r}")
                 return parsed_version
+        return None
 
     @cached_property
-    def supported(self):
+    def supported(self) -> bool:
         """Is the package manager supported on that platform?"""
         return CURRENT_OS_ID in self.platforms
 
     @cached_property
-    def executable(self):
+    def executable(self) -> bool:
         """Is the package manager CLI can be executed by the current user?"""
         if not self.cli_path:
             return False
@@ -317,7 +448,7 @@ class PackageManager:
         return True
 
     @cached_property
-    def fresh(self):
+    def fresh(self) -> bool:
         """Does the package manager match the version requirement?"""
         # Version is mandatory.
         if not self.version:
@@ -332,7 +463,7 @@ class PackageManager:
         return True
 
     @cached_property
-    def available(self):
+    def available(self) -> bool:
         """Is the package manager available and ready-to-use on the system?
 
         Returns ``True`` only if the main CLI:
@@ -350,16 +481,7 @@ class PackageManager:
         )
         return bool(self.supported and self.cli_path and self.executable and self.fresh)
 
-    @classmethod
-    def args_cleanup(cls, *args):
-        """Flatten recursive iterables, remove all ``None``, and cast each element to
-        strings.
-
-        Helps serialize :py:class:`pathlib.Path` and :py:class:`meta_package_manager.version.TokenizedString` objects.
-        """
-        return tuple(map(str, filter(None.__ne__, flatten(args))))
-
-    def run(self, *args, extra_env=None):
+    def run(self, *args: _Arg | _NestedArgs, extra_env: _EnvVars = None) -> str:
         """Run a shell command, return the output and accumulate error messages.
 
         ``args`` is allowed to be a nested structure of iterables, in which case it will
@@ -377,8 +499,8 @@ class PackageManager:
             Move :option:`mpm --dry-run` option and this method to `click-extra <https://github.com/kdeldycke/click-extra>`_.
         """
         # Casting to string helps serialize Path and Version objects.
-        args = self.args_cleanup(args)
-        cli_msg = format_cli(args, extra_env)
+        clean_args = ExtraCliRunner._args_cleanup(*args)
+        cli_msg = format_cli(clean_args, extra_env)
 
         code = 0
         output = ""
@@ -389,7 +511,7 @@ class PackageManager:
         else:
             logger.debug(cli_msg)
             code, output, error = run_cmd(
-                *args, extra_env=extra_env, print_output=False
+                *clean_args, extra_env=extra_env, print_output=False
             )
 
         # Normalize messages.
@@ -419,16 +541,16 @@ class PackageManager:
 
     def build_cli(
         self,
-        *args,
-        auto_pre_cmds=True,
-        auto_pre_args=True,
-        auto_post_args=True,
-        override_pre_cmds=False,
-        override_cli_path=False,
-        override_pre_args=False,
-        override_post_args=False,
-        sudo=False,
-    ):
+        *args: _Arg | _NestedArgs,
+        auto_pre_cmds: bool = True,
+        auto_pre_args: bool = True,
+        auto_post_args: bool = True,
+        override_pre_cmds: _NestedArgs | None = None,
+        override_cli_path: Path | None = None,
+        override_pre_args: _NestedArgs | None = None,
+        override_post_args: _NestedArgs | None = None,
+        sudo: bool = False,
+    ) -> tuple[str, ...]:
         """Build the package manager CLI by combining the custom ``*args`` with the
         package manager's global parameters.
 
@@ -455,7 +577,7 @@ class PackageManager:
         * ``auto_pre_args=False``  to skip the automatic addition of :py:attr:`self.pre_args <meta_package_manager.base.PackageManager.pre_args>`
         * ``auto_post_args=False`` to skip the automatic addition of :py:attr:`self.post_args <meta_package_manager.base.PackageManager.post_args>`
 
-        Each global set of elements can be locally overriden with:
+        Each global set of elements can be locally overridden with:
 
         * ``override_pre_cmds=tuple()``
         * ``override_cli_path=str``
@@ -465,9 +587,9 @@ class PackageManager:
         On linux, the command can be run with `sudo <https://www.sudo.ws>`_ if the parameter of the same name is set to ``True``.
         In which case the ``override_pre_cmds`` parameter is not allowed to be set and the ``auto_pre_cmds`` parameter is forced to ``False``.
         """
-        params = []
+        params: list[_Arg | _NestedArgs] = []
 
-        # Sudo replaces any pre-command, be it overriden or automatic.
+        # Sudo replaces any pre-command, be it overridden or automatic.
         if sudo:
             if not is_linux():
                 raise NotImplementedError("sudo only supported on Linux.")
@@ -477,19 +599,16 @@ class PackageManager:
                 auto_pre_cmds = False
             params.append("sudo")
         elif override_pre_cmds:
-            assert isinstance(override_pre_cmds, tuple)
             params.extend(override_pre_cmds)
         elif auto_pre_cmds:
             params.extend(self.pre_cmds)
 
         if override_cli_path:
-            assert isinstance(override_pre_cmds, str)
             params.append(override_cli_path)
         else:
             params.append(self.cli_path)
 
         if override_pre_args:
-            assert isinstance(override_pre_args, tuple)
             params.extend(override_pre_args)
         elif auto_pre_args:
             params.extend(self.pre_args)
@@ -498,28 +617,27 @@ class PackageManager:
             params.extend(args)
 
         if override_post_args:
-            assert isinstance(override_post_args, tuple)
             params.extend(override_post_args)
         elif auto_post_args:
             params.extend(self.post_args)
 
-        return self.args_cleanup(params)
+        return ExtraCliRunner._args_cleanup(cast(_NestedArgs, params))
 
     def run_cli(
         self,
-        *args,
-        auto_extra_env=True,
-        auto_pre_cmds=True,
-        auto_pre_args=True,
-        auto_post_args=True,
-        override_extra_env=False,
-        override_pre_cmds=False,
-        override_cli_path=False,
-        override_pre_args=False,
-        override_post_args=False,
-        force_exec=False,
-        sudo=False,
-    ):
+        *args: _Arg | _NestedArgs,
+        auto_extra_env: bool = True,
+        auto_pre_cmds: bool = True,
+        auto_pre_args: bool = True,
+        auto_post_args: bool = True,
+        override_extra_env: _EnvVars = None,
+        override_pre_cmds: _NestedArgs | None = None,
+        override_cli_path: Path | None = None,
+        override_pre_args: _NestedArgs | None = None,
+        override_post_args: _NestedArgs | None = None,
+        force_exec: bool = False,
+        sudo: bool = False,
+    ) -> str:
         """Build and run the package manager CLI by combining the custom ``*args`` with
         the package manager's global parameters.
 
@@ -549,13 +667,13 @@ class PackageManager:
         # Prepare the full list of CLI arguments.
         extra_env = None
         if override_extra_env:
-            assert isinstance(override_extra_env, dict)
             extra_env = override_extra_env
         elif auto_extra_env:
             extra_env = self.extra_env
 
         # No-op context manager without any effects.
-        local_option1 = local_option2 = nullcontext()
+        local_option1: ContextManager = nullcontext()
+        local_option2: ContextManager = nullcontext()
         # Temporarily replace --dry-run and --stop-on-error user options with our own.
         if force_exec:
             local_option1 = patch.object(self, "dry_run", False)
@@ -566,105 +684,38 @@ class PackageManager:
 
         return output
 
+    def package(self, *args, **kwargs) -> Package:
+        """Produce a ``Package`` object with provided attributes."""
+        kwargs["manager"] = self
+        return self.package_class(*args, **kwargs)
+
     @property
-    def installed(self):
+    def installed(self) -> Iterator[Package]:
         """List packages currently installed on the system.
 
-        Returns a ``dict`` indexed by package ``id``. Each item is a ``dict`` with:
-
-        - (repeating) package's ``id``
-        - ``name`` (often the same as ``id``)
-        - current ``installed_version``
-
-        .. code-block:: python
-
-            {
-                "ack": {
-                    "id": "ack",
-                    "name": "ack",
-                    "installed_version": "3.5.0",
-                },
-                "aom": {
-                    "id": "aom",
-                    "name": "aom",
-                    "installed_version": "3.3.0",
-                },
-                (...)
-            }
-
         Optional. Will be simply skipped by :program:`mpm` if not implemented.
         """
         raise NotImplementedError
 
     @property
-    def outdated(self):
+    def outdated(self) -> Iterator[Package]:
         """List installed packages with available upgrades.
 
-        Returns a ``dict`` indexed by package ``id``. Each item is a ``dict`` with:
-
-        - (repeating) package's ``id``
-        - ``name`` (often the same as ``id``)
-        - current ``installed_version``
-        - ``latest_version`` available for upgrade
-
-        .. code-block:: python
-
-            {
-                "awscli": {
-                    "id": "awscli",
-                    "name": "awscli",
-                    "installed_version": "2.5.6",
-                    "latest_version": "2.5.8",
-                },
-                "git": {
-                    "id": "git",
-                    "name": "git",
-                    "installed_version": "2.35.3",
-                    "latest_version": "2.36.0",
-                },
-                (...)
-            }
-
         Optional. Will be simply skipped by :program:`mpm` if not implemented.
         """
         raise NotImplementedError
 
     @classmethod
-    def query_parts(cls, query):
+    def query_parts(cls, query: str) -> set[str]:
         """Returns a set of all contiguous alphanumeric string segments.
 
         Contrary to :py:class:`meta_package_manager.version.TokenizedString`, do no splits on colated number/alphabetic junctions.
         """
         return {p for p in re.split(r"\W+", query) if p}
 
-    def search(self, query, extended, exact):
+    def search(self, query: str, extended: bool, exact: bool) -> Iterator[Package]:
         """Search packages available for install.
 
-        Returns a generator yielding ``dict`` items containing:
-
-        - the package's ``id`` as a string
-        - its ``name`` (often the same as ``id``) as a string
-        - ``description`` as a string or ``None`` if not available
-        - ``latest_version`` available for install, as a :py:class:`meta_package_manager.version.TokenizedString` instance or ``None``
-
-        .. code-block:: python
-
-            (
-                {
-                    "id": "google",
-                    "name": "google",
-                    "description": "Library to interact with Google.",
-                    "latest_version": <TokenizedString 2.1.0 => ('2', '1', '0')>,
-                },
-                {
-                    "id": "@google-cloud/storage",
-                    "name": "Google cloud storage",
-                    "description": None,
-                    "latest_version": None,
-                },
-                (...)
-            )
-
         There is no need for this method to be perfect and sensitive to ``extended`` and ``exact`` parameters. If the package manager
         is not supporting these kind of options out of the box, just returns the closest subset of matching package you can
         come up with. Finer refiltering will happens in the :py:meth:`meta_package_manager.base.PackageManager.refiltered_search` method below.
@@ -673,7 +724,9 @@ class PackageManager:
         """
         raise NotImplementedError
 
-    def refiltered_search(self, query, extended, exact):
+    def refiltered_search(
+        self, query: str, extended: bool, exact: bool
+    ) -> Iterator[Package]:
         """Returns search results with extra manual refiltering to refine gross
         matchings.
 
@@ -699,7 +752,7 @@ class PackageManager:
             # Look by default into package ID and name.
             search_content = {match.id, match.name}
 
-            # Rejects fuzzy results: only keep packages stricly matching on ID or name.
+            # Rejects fuzzy results: only keep packages strictly matching on ID or name.
             if exact and query not in search_content:
                 continue
 
@@ -708,12 +761,12 @@ class PackageManager:
                 search_content.add(match.description)
 
             # Normalize searched content.
-            search_content = "".join({s.lower() for s in search_content if s})
+            serialized_content = "".join({s.lower() for s in search_content if s})
 
             # Exclude packages not matching any part of the query.
             confirmed_match = False
             for part in {p.lower() for p in self.query_parts(query)}:
-                if part in search_content:
+                if part in serialized_content:
                     confirmed_match = True
                     break
             if not confirmed_match:
@@ -722,83 +775,70 @@ class PackageManager:
             # Report the package as matching.
             yield match
 
-    def install(self, package_id):
+    def install(self, package_id: str) -> str:
         """Install one package and one only.
 
         Optional. Will be simply skipped by :program:`mpm` if not implemented.
         """
         raise NotImplementedError
 
-    def upgrade_cli(self, package_id=None):
-        """Returns the complete CLI to upgrade the package provided as ``package_id``
-        parameter.
-
-        If ``package_id`` is ``None``, the returned CLI is expected be the one that is
-        used to upgrade all outdated packages on the system.
-        """
+    def upgrade_all_cli(self) -> tuple[str, ...]:
+        """Returns the complete CLI to upgrade all outdated packages on the system."""
         raise NotImplementedError
 
-    def upgrade(self, package_id=None):
-        """Perform the upgrade of the provided package to latest version.
+    def upgrade_one_cli(self, package_id: str) -> tuple[str, ...]:
+        """Returns the complete CLI to upgrade one package and one only."""
+        raise NotImplementedError
 
-        Executes the CLI provided by :py:meth:`meta_package_manager.base.PackageManager.upgrade_cli`.
-        """
-        return self.run(self.upgrade_cli(package_id), extra_env=self.extra_env)
+    def upgrade(self, package_id: str | None = None) -> str:
+        """Perform an upgrade of either all or one package.
 
-    def upgrade_all_cli(self):
-        """Returns the complete CLI to upgrade all outdated packages on the system.
-
-        By default, returns the result of the :py:meth:`meta_package_manager.base.PackageManager.upgrade_cli`
-        method, without a ``package_id`` parameter.
-
-        This dedicated method allows some package manager to return a CLI that is very different from the
-        one returned by :py:meth:`meta_package_manager.base.PackageManager.upgrade_cli`.
-
-        It can also be used to ``raise NotImplementedError`` on some managers to signal to :program:`mpm` that
-        the package manager has no proper support for a full upgrade operation. See for example
-        :py:meth:`meta_package_manager.managers.pip.Pip.upgrade_all_cli`.
-        """
-        return self.upgrade_cli()
-
-    def upgrade_all(self):
-        """Perform a full upgrade of all outdated packages on the system.
-
-        Executes the CLI provided by :py:meth:`meta_package_manager.base.PackageManager.upgrade_all_cli`.
+        Executes the CLI provided by either
+        :py:meth:`meta_package_manager.base.PackageManager.upgrade_all_cli` or
+        :py:meth:`meta_package_manager.base.PackageManager.upgrade_one_cli`.
 
         If the manager doesn't provides a full upgrade one-liner (i.e. if
         :py:meth:`meta_package_manager.base.PackageManager.upgrade_all_cli` raises
         :py:exc:`NotImplementedError`), then the list of all outdated packages will be fetched
         (via :py:meth:`meta_package_manager.base.PackageManager.outdated`) and each package will be
-        updated one by one by a :py:meth:`meta_package_manager.base.PackageManager.upgrade` call.
+        updated one by one by calling :py:meth:`meta_package_manager.base.PackageManager.upgrade_one_cli`.
+
+        See for example the case of :py:meth:`meta_package_manager.managers.pip.Pip.upgrade_one_cli`.
         """
-        try:
-            return self.run(self.upgrade_all_cli(), extra_env=self.extra_env)
-        except NotImplementedError:
-            logger.warning(f"{self.id} does not implement upgrade_all operation.")
+        if package_id:
+            cli = self.upgrade_one_cli(package_id)
 
-        logger.info(f"Fallback to calling upgrade operation on each outdated package.")
-        log = ""
-        for package_id in self.outdated:
-            output = self.upgrade(package_id)
-            if output:
-                log += f"\n{output}"
-        return log
+        else:
+            try:
+                cli = self.upgrade_all_cli()
+            except NotImplementedError:
+                logger.info(
+                    f"Fallback to calling upgrade operation on each outdated package."
+                )
+                log = ""
+                for package in self.outdated:
+                    output = self.upgrade(package.id)
+                    if output:
+                        log += f"\n{output}"
+                return log
 
-    def remove(self, package_id):
+        return self.run(cli, extra_env=self.extra_env)
+
+    def remove(self, package_id: str) -> str:
         """Remove one package and one only.
 
         Optional. Will be simply skipped by :program:`mpm` if not implemented.
         """
         raise NotImplementedError
 
-    def sync(self):
+    def sync(self) -> None:
         """Refresh package metadata from remote repositories.
 
         Optional. Will be simply skipped by :program:`mpm` if not implemented.
         """
         raise NotImplementedError
 
-    def cleanup(self):
+    def cleanup(self) -> None:
         """Removes left-overs, orphaned dependencies,
 
         Optional. Will be simply skipped by :program:`mpm` if not implemented.
