@@ -15,30 +15,25 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-import dataclasses
+from __future__ import annotations
+
 import logging
 import sys
-from collections import namedtuple
+from collections import Counter, namedtuple
+from contextlib import contextmanager
 from datetime import datetime
 from functools import partial
 from io import TextIOWrapper
 from operator import attrgetter
 from pathlib import Path
+from typing import Iterator
 from unittest.mock import patch
 
 import tomli_w
 from boltons.cacheutils import LRI, cached
-from click_extra import (
-    STRING,
-    Choice,
-    File,
-    argument,
-    echo,
-    group,
-    option,
-    option_group,
-    pass_context,
-)
+from click_extra import STRING, Choice, File, Option
+from click_extra import Path as ClickPath
+from click_extra import argument, echo, extra_group, option, option_group, pass_context
 from click_extra.colorize import KO, OK, highlight, theme
 from click_extra.platform import os_label
 from click_extra.tabulate import table_format_option
@@ -51,7 +46,7 @@ else:
 from cloup import Section
 
 from . import __version__, bar_plugin, logger
-from .base import CLIError, PackageManager
+from .base import CLIError, Operations, PackageManager, packages_asdict
 from .output import (
     SORTABLE_FIELDS,
     BarPluginRenderer,
@@ -65,7 +60,7 @@ from .pool import pool
 
 # Subcommand sections.
 EXPLORE = Section("Explore subcommands")
-MAINTENANCE = Section("Maintainance subcommands")
+MAINTENANCE = Section("Maintenance subcommands")
 SNAPSHOTS = Section("Package snapshots subcommands")
 
 
@@ -81,14 +76,14 @@ def add_manager_to_selection(ctx, param, selected):
     if selected:
         if ctx.obj is None:
             ctx.obj = {"single_manager_selector": []}
-        # Parameter's name is transformed into a Python identifier on instanciation.
+        # Parameter's name is transformed into a Python identifier on instantiation.
         # Reverse the process to get our value.
         # Example: "--apt-mint" => "apt_mint" => "apt-mint"
         manager_id = param.name.replace("_", "-")
         ctx.obj["single_manager_selector"].append(manager_id)
 
 
-def single_manager_selectors():
+def single_manager_selectors() -> Iterator[Option]:
     """Dynamiccaly creates a dedicated flag selector alias for each manager."""
     for manager_id in pool.all_manager_ids:
         yield option(
@@ -131,7 +126,10 @@ def bar_plugin_path(ctx, param, value):
         ctx.exit()
 
 
-@group(version=__version__)
+# XXX Why is Sphinx skipping Click methods in documentation?
+
+
+@extra_group(version=__version__)
 @option_group(
     "Package manager selection options",
     option(
@@ -139,7 +137,7 @@ def bar_plugin_path(ctx, param, value):
         "--manager",
         type=Choice(pool.all_manager_ids, case_sensitive=False),
         multiple=True,
-        help="Restrict subcommand to a subset of package managers. Repeat to "
+        help="Restrict subcommand to a subset of managers. Repeat to "
         "select multiple managers. The order in which options are provided defines the "
         "order in which subcommands will process them.",
     ),
@@ -148,14 +146,14 @@ def bar_plugin_path(ctx, param, value):
         "--exclude",
         type=Choice(pool.all_manager_ids, case_sensitive=False),
         multiple=True,
-        help="Exclude a package manager. Repeat to exclude multiple managers.",
+        help="Exclude a manager. Repeat to exclude multiple managers.",
     ),
     option(
         "-a",
         "--all-managers",
         is_flag=True,
         default=False,
-        help="Force evaluation of all package manager recognized by mpm, even those "
+        help="Force evaluation of all manager recognized by mpm, even those "
         "not supported by the current platform. Still applies filtering by --manager "
         "and --exclude options before calling the subcommand.",
     ),
@@ -164,8 +162,8 @@ def bar_plugin_path(ctx, param, value):
         "--xkcd",
         is_flag=True,
         default=False,
-        help="Preset manager selection as defined by XKCD #1654, i.e. "
-        "{}.".format(", ".join(map(theme.choice, XKCD_MANAGER_ORDER))),
+        help="Preset manager selection as defined by XKCD #1654. Equivalent to: "
+        "{}.".format(" ".join(f"--{mid}" for mid in XKCD_MANAGER_ORDER)),
     ),
     *single_manager_selectors(),
 )
@@ -200,6 +198,12 @@ def bar_plugin_path(ctx, param, value):
         ),
         help="Rendering mode of the output.",
     ),
+    option(
+        "--description",
+        is_flag=True,
+        default=False,
+        help="Show package description in results.",
+    ),
     option(
         "-s",
         "--sort-by",
@@ -233,10 +237,11 @@ def mpm(
     ignore_auto_updates,
     stop_on_error,
     dry_run,
+    description,
     sort_by,
     stats,
 ):
-    """Common CLI options and behavior for multiple package managers."""
+    """Common CLI options and behavior for managers."""
 
     # Update the list of selected managers with single selectors.
     if ctx.obj:
@@ -249,13 +254,11 @@ def mpm(
         logger.setLevel(logging.CRITICAL * 2)
 
     # Select the subset of manager to target, and apply manager-level options.
-    selected_managers = pool.select_managers(
+    selected_managers = partial(
+        pool.select_managers,
         keep=manager if not xkcd else XKCD_MANAGER_ORDER,
         drop=exclude,
-        drop_unsupported=not all_managers,
-        # Only keep inactive managers to show them in the "managers" subcommand table.
-        # Filters them out in any other subcommand.
-        drop_inactive=ctx.invoked_subcommand != "managers",
+        keep_unsupported=all_managers,
         # Should we include auto-update packages or not?
         ignore_auto_updates=ignore_auto_updates,
         # Does the manager should raise on error or not.
@@ -268,11 +271,13 @@ def mpm(
         "GlobalOptions",
         (
             "selected_managers",
+            "description",
             "sort_by",
             "stats",
         ),
         defaults=(
             selected_managers,
+            description,
             sort_by,
             stats,
         ),
@@ -286,6 +291,12 @@ def mpm(
 def managers(ctx):
     """List all supported package managers and autodetect their presence on the
     system."""
+    select_params = {
+        # Do not drop inactive managers. Keep them to show off how mpm is reacting
+        # to the local platform.
+        "drop_inactive": False,
+    }
+
     # Machine-friendly data rendering.
     if ctx.find_root().table_formatter.format_name == "json":
         manager_data = {}
@@ -300,7 +311,7 @@ def managers(ctx):
             "fresh",
             "available",
         )
-        for manager in ctx.obj.selected_managers:
+        for manager in ctx.obj.selected_managers(**select_params):
             manager_data[manager.id] = {fid: getattr(manager, fid) for fid in fields}
             # Serialize errors at the last minute to gather all we encountered.
             manager_data[manager.id]["errors"] = list(
@@ -308,11 +319,11 @@ def managers(ctx):
             )
 
         print_json(manager_data)
-        return
+        ctx.exit()
 
     # Human-friendly content rendering.
     table = []
-    for manager in ctx.obj.selected_managers:
+    for manager in ctx.obj.selected_managers(**select_params):
 
         # Build up the OS column content.
         os_infos = OK if manager.supported else KO
@@ -324,9 +335,11 @@ def managers(ctx):
         # Build up the CLI path column content.
         cli_infos = "{} {}".format(
             OK if manager.cli_path else KO,
+            # TODO: highlight cli_name found in manager.cli_path. I.e, the "choco" string in:
+            #    ✓ C:\ProgramData\chocolatey\bin\choco.EXE
             manager.cli_path
             if manager.cli_path
-            else f"{', '.join(manager.cli_names)} not found",
+            else f"{', '.join(map(theme.invoked_command, manager.cli_names))} not found",
         )
 
         # Build up the version column content.
@@ -340,8 +353,8 @@ def managers(ctx):
 
         table.append(
             (
-                manager.name,
                 getattr(theme, "success" if manager.fresh else "error")(manager.id),
+                manager.name,
                 os_infos,
                 cli_infos,
                 OK if manager.executable else "",
@@ -351,8 +364,8 @@ def managers(ctx):
 
     print_table(
         (
-            ("Package manager", "manager_name"),
-            ("ID", "manager_id"),
+            ("Manager ID", "manager_id"),
+            ("Name", "manager_name"),
             ("Supported", None),
             ("CLI", None),
             ("Executable", None),
@@ -363,20 +376,29 @@ def managers(ctx):
     )
 
 
-@mpm.command(short_help="List installed packages.", section=EXPLORE)
+@mpm.command(aliases=["list"], short_help="List installed packages.", section=EXPLORE)
+@option(
+    "-d",
+    "--duplicates",
+    is_flag=True,
+    default=False,
+    help="Only list installed packages sharing the same ID. Implies "
+    "`--sort_by package_id` to make duplicates easier to compare between themselves.",
+)
 @pass_context
-def installed(ctx):
-    """List all packages installed on the system from all managers."""
+def installed(ctx, duplicates):
+    """List all packages installed on the system by each manager."""
     # Build-up a global dict of installed packages per manager.
     installed_data = {}
+    fields = (
+        "id",
+        "name",
+        "installed_version",
+    )
 
-    for manager in ctx.obj.selected_managers:
+    for manager in ctx.obj.selected_managers(implements_operation=Operations.installed):
 
-        try:
-            packages = tuple(map(dataclasses.asdict, manager.installed))
-        except NotImplementedError:
-            logger.warning(f"{manager.id} does not implement installed operation.")
-            continue
+        packages = tuple(packages_asdict(manager.installed, fields))
 
         installed_data[manager.id] = {
             "id": manager.id,
@@ -389,38 +411,68 @@ def installed(ctx):
             {expt.error for expt in manager.cli_errors}
         )
 
+    # Filters out non-duplicate packages.
+    if duplicates:
+        # Re-group packages by their IDs.
+        package_sources = {}
+        for manager_id, installed_pkg in installed_data.items():
+            for package in installed_pkg["packages"]:
+                package_sources.setdefault(package["id"], set()).add(manager_id)
+        logger.debug(f"Managers sourcing each package: {package_sources}")
+
+        # Identify package IDs shared by multiple managers.
+        duplicates_ids = {
+            pid for pid, managers in package_sources.items() if len(managers) > 1
+        }
+        logger.debug(f"Duplicates: {duplicates_ids}")
+
+        # Remove non-duplicates from results.
+        for manager_id in installed_data.keys():
+            duplicate_packages = tuple(
+                p
+                for p in installed_data[manager_id]["packages"]
+                if p["id"] in duplicates_ids
+            )
+            installed_data[manager_id]["packages"] = duplicate_packages
+
     # Machine-friendly data rendering.
     if ctx.find_root().table_formatter.format_name == "json":
         print_json(installed_data)
-        return
+        ctx.exit()
 
     # Human-friendly content rendering.
     table = []
     for manager_id, installed_pkg in installed_data.items():
         table += [
             (
-                info["name"],
                 info["id"],
+                info["name"],
                 manager_id,
                 info["installed_version"] if info["installed_version"] else "?",
             )
             for info in installed_pkg["packages"]
         ]
 
-    # Sort and print table.
+    # Force sorting by package ID in duplicate mode.
+    sort_by = ctx.obj.sort_by
+    if duplicates:
+        logger.info("Force table sorting on package ID because of --duplicates option.")
+        sort_by = "package_id"
+
+    # Print table.
     print_table(
         (
-            ("Package name", "package_name"),
-            ("ID", "package_id"),
+            ("Package ID", "package_id"),
+            ("Name", "package_name"),
             ("Manager", "manager_id"),
             ("Installed version", "version"),
         ),
         table,
-        ctx.obj.sort_by,
+        sort_by,
     )
 
     if ctx.obj.stats:
-        print_stats(installed_data)
+        print_stats(Counter({k: len(v["packages"]) for k, v in installed_data.items()}))
 
 
 @mpm.command(short_help="List outdated packages.", section=EXPLORE)
@@ -428,32 +480,34 @@ def installed(ctx):
     "--plugin-output",
     is_flag=True,
     default=False,
-    help="Output results for direct consumption by an Xbar/SwiftBar-compatible plugin."
-    " The layout is dynamic and depends on environment variables set by both Xbar and "
-    "SwiftBar.",
+    help="Output results for direct consumption by an Xbar/SwiftBar-compatible plugin. "
+    "The layout is dynamic and depends on environment variables set by either Xbar "
+    "or SwiftBar.",
 )
 @pass_context
 def outdated(ctx, plugin_output):
     """List available package upgrades and their versions for each manager."""
     # Build-up a global list of outdated packages per manager.
     outdated_data = {}
+    fields = [
+        "id",
+        "name",
+        "installed_version",
+        "latest_version",
+    ]
+    if plugin_output:
+        fields.extend(["label", "upgrade_cli"])
 
-    for manager in ctx.obj.selected_managers:
+    for manager in ctx.obj.selected_managers(implements_operation=Operations.outdated):
 
-        try:
-            packages = tuple(map(dataclasses.asdict, manager.outdated))
-        except NotImplementedError:
-            logger.warning(f"{manager.id} does not implement outdated operation.")
-            continue
+        packages = tuple(packages_asdict(manager.outdated, tuple(fields)))
 
-        for info in packages:
-            info.update(
-                {
-                    "upgrade_cli": BarPluginRenderer.render_cli(
-                        manager.upgrade_cli(info["id"]), plugin_format=plugin_output
-                    )
-                }
-            )
+        # Re-render upgrade CLI for plugin consumption.
+        if plugin_output:
+            for info in packages:
+                info["upgrade_cli"] = BarPluginRenderer.render_cli(
+                    info["upgrade_cli"], plugin_format=plugin_output
+                )
 
         outdated_data[manager.id] = {
             "id": manager.id,
@@ -461,18 +515,15 @@ def outdated(ctx, plugin_output):
             "packages": packages,
         }
 
-        # Do not include the full-upgrade CLI if we did not detect any outdated
-        # package.
-        if packages:
+        # Only include full-upgrade CLI for plugin.
+        if plugin_output and packages:
             try:
                 upgrade_all_cli = manager.upgrade_all_cli()
             except NotImplementedError:
                 # Fallback on mpm itself which is capable of simulating a full upgrade.
-                logger.warning(
-                    f"{manager.id} does not implement upgrade_all operation."
-                )
+                logger.warning(f"{manager.id} does not implement upgrade_all_cli.")
                 mpm_exec = bar_plugin.MPMPlugin().mpm_exec
-                upgrade_all_cli = (*mpm_exec, f"--{manager.id}", "upgrade")
+                upgrade_all_cli = (*mpm_exec, f"--{manager.id}", "upgrade", "--all")
                 logger.debug(f"Fallback to direct mpm call: {upgrade_all_cli}")
             outdated_data[manager.id]["upgrade_all_cli"] = BarPluginRenderer.render_cli(
                 upgrade_all_cli, plugin_format=plugin_output
@@ -486,12 +537,12 @@ def outdated(ctx, plugin_output):
     # Machine-friendly data rendering.
     if ctx.find_root().table_formatter.format_name == "json":
         print_json(outdated_data)
-        return
+        ctx.exit()
 
     # Xbar/SwiftBar-friendly plugin rendering.
     if plugin_output:
         BarPluginRenderer().print(outdated_data)
-        return
+        ctx.exit()
 
     # Human-friendly content rendering.
     table = []
@@ -503,8 +554,8 @@ def outdated(ctx, plugin_output):
             )
             table.append(
                 (
-                    info["name"],
                     info["id"],
+                    info["name"],
                     manager_id,
                     installed_version,
                     latest_version,
@@ -514,8 +565,8 @@ def outdated(ctx, plugin_output):
     # Sort and print table.
     print_table(
         (
-            ("Package name", "package_name"),
-            ("ID", "package_id"),
+            ("Package ID", "package_id"),
+            ("Name", "package_name"),
             ("Manager", "manager_id"),
             ("Installed version", "version"),
             ("Latest version", None),
@@ -525,56 +576,67 @@ def outdated(ctx, plugin_output):
     )
 
     if ctx.obj.stats:
-        print_stats(outdated_data)
+        print_stats(Counter({k: len(v["packages"]) for k, v in outdated_data.items()}))
 
 
 # TODO: make it a --search-strategy=[exact, fuzzy, extended]
 # Add details helps => exact: is case-sensitive, and keep all non-alnum chars
 # fuzzy: query is case-insensitive, stripped-out of non-alnum chars and
 # tokenized (no order sensitive)
-# extended, same as fuzzy, but do not limit search to package name and ID.
+# extended, same as fuzzy, but do not limit search to package ID and name.
 # extended to description and other metadata depending on manager support.
 # Modes:
-#  1. strict (--exact)
+#  1. strict (--exact, on ID or name)
 #  2. substring (regex, no case, no split)
 #  3. fuzzy (token-based)
-#  4. extended (+ metadata)
+#  4. extended (fuzzy + metadata)
 @mpm.command(short_help="Search packages.", section=EXPLORE)
 @option(
-    "--extended/--package-name",
+    "--extended/--id-name-only",
     default=False,
-    help="Extend search to additional package metadata like description, "
-    "instead of restricting it package ID and name.",
+    help="Extend search to description, instead of restricting it to package ID and name. Implies --description.",
 )
 @option(
     "--exact/--fuzzy",
     default=False,
-    help="Only returns exact matches, or enable fuzzy search in substrings.",
+    help="Only returns exact matches on package ID or name.",
+)
+@option(
+    "--refilter/--no-refilter",
+    default=True,
+    help="Let mpm refilters managers' search results.",
 )
 @argument("query", type=STRING, required=True)
 @pass_context
-def search(ctx, extended, exact, query):
-    """Search packages from all managers."""
+def search(ctx, extended, exact, refilter, query):
+    """Search each manager for a package ID, name or description matching the query."""
+    # --extended implies --description.
+    show_description = ctx.obj.description
+    if extended and not ctx.obj.description:
+        logger.warning("--extended option forces --description option.")
+        show_description = True
+
     # Build-up a global list of package matches per manager.
     matches = {}
+    fields = (
+        "id",
+        "name",
+        "latest_version",
+        "description",
+    )
 
-    for manager in ctx.obj.selected_managers:
-
-        try:
-            results = tuple(
-                map(
-                    dataclasses.asdict,
-                    manager.refiltered_search(query, extended, exact),
-                )
+    search_method = "refiltered_search" if refilter else "search"
+    for manager in ctx.obj.selected_managers(implements_operation=Operations.search):
+        packages = tuple(
+            packages_asdict(
+                getattr(manager, search_method)(query, extended, exact), fields
             )
-        except NotImplementedError:
-            logger.warning(f"{manager.id} does not implement search operation.")
-            continue
+        )
 
         matches[manager.id] = {
             "id": manager.id,
             "name": manager.name,
-            "packages": results,
+            "packages": packages,
         }
 
         # Serialize errors at the last minute to gather all we encountered.
@@ -585,7 +647,7 @@ def search(ctx, extended, exact, query):
     # Machine-friendly data rendering.
     if ctx.find_root().table_formatter.format_name == "json":
         print_json(matches)
-        return
+        ctx.exit()
 
     # Prepare highlighting helpers.
     query_parts = {query}.union(PackageManager.query_parts(query))
@@ -601,30 +663,34 @@ def search(ctx, extended, exact, query):
     # Human-friendly content rendering.
     table = []
     for manager_id, matching_pkg in matches.items():
-        table += [
-            (
-                highlight_query(info["name"]),
-                highlight_query(info["id"]),
+        for pkg in matching_pkg["packages"]:
+            line = [
+                highlight_query(pkg["id"]) if pkg["id"] else "",
+                highlight_query(pkg["name"]) if pkg["name"] else "",
                 manager_id,
-                info["latest_version"] if info["latest_version"] else "?",
-            )
-            for info in matching_pkg["packages"]
-        ]
+                pkg["latest_version"] if pkg["latest_version"] else "?",
+            ]
+            if show_description:
+                line.append(
+                    highlight_query(pkg.get("description"))
+                    if pkg.get("description")
+                    else ""
+                )
+            table.append(line)
 
     # Sort and print table.
-    print_table(
-        (
-            ("Package name", "package_name"),
-            ("ID", "package_id"),
-            ("Manager", "manager_id"),
-            ("Latest version", "version"),
-        ),
-        table,
-        ctx.obj.sort_by,
-    )
+    headers = [
+        ("Package ID", "package_id"),
+        ("Name", "package_name"),
+        ("Manager", "manager_id"),
+        ("Latest version", "version"),
+    ]
+    if show_description:
+        headers.append(("Description", "description"))
+    print_table(headers, table, ctx.obj.sort_by)
 
     if ctx.obj.stats:
-        print_stats(matches)
+        print_stats(Counter({k: len(v["packages"]) for k, v in matches.items()}))
 
 
 @mpm.command(short_help="Install a package.", section=MAINTENANCE)
@@ -632,9 +698,11 @@ def search(ctx, extended, exact, query):
 # TODO: add a --force/--reinstall flag
 @pass_context
 def install(ctx, package_id):
-    """Install the provided package using one of the selected package manager."""
+    """Install the provided package ID using one of the selected manager."""
     # Cast generator to tuple because of reuse.
-    selected_managers = tuple(ctx.obj.selected_managers)
+    selected_managers = tuple(
+        ctx.obj.selected_managers(implements_operation=Operations.install)
+    )
 
     logger.info(
         f"Package manager order: {', '.join([m.id for m in selected_managers])}"
@@ -662,7 +730,7 @@ def install(ctx, package_id):
             if len(matches) != 1:
                 raise ValueError("Exact search returned multiple packages.")
 
-        # Allow install subcommand to fail to have the oportunity to catch the CLIError exception and print
+        # Allow install subcommand to fail to have the opportunity to catch the CLIError exception and print
         # a comprehensive message.
         with patch.object(manager, "stop_on_error", True):
             try:
@@ -676,32 +744,105 @@ def install(ctx, package_id):
                 continue
 
         echo(output)
-        return
+        ctx.exit()
 
 
-@mpm.command(short_help="Upgrade all packages.", section=MAINTENANCE)
+@mpm.command(aliases=["update"], short_help="Upgrade packages.", section=MAINTENANCE)
+@option(
+    "-A",
+    "--all",
+    is_flag=True,
+    default=False,
+    help="Upgrade all outdated packages. "
+    "Will make the command ignore package IDs provided as parameters.",
+)
+@argument("package_ids", type=STRING, nargs=-1)
 @pass_context
-def upgrade(ctx):
-    """Perform a full package upgrade on all available managers."""
-    for manager in ctx.obj.selected_managers:
-        logger.info(f"Updating all outdated packages from {manager.id}...")
-        try:
-            output = manager.upgrade_all()
-        except NotImplementedError:
-            logger.warning(f"{manager.id} does not implement upgrade_all operation.")
+def upgrade(ctx, all, package_ids):
+    """Upgrade one or more outdated packages.
+
+    Defaults to upgrading all outdated package if none provided as arguments (i.e. assumes
+    -A/--all if no [PACKAGE_IDS]).
+
+    Upgrade will only proceed if no ambiguity is uncovered. Packages recognized by multiple
+    managers will be skipped. You can remove that ambiguity by carefully specifying the subset of
+    managers to consider for upgrade.
+
+    Unknown packages will be skipped.
+    """
+    if not all and not package_ids:
+        logger.warning("No package provided, assume -A/--all option.")
+        all = True
+
+    # Deduplicate entries.
+    package_ids = set(package_ids)
+
+    # Full upgrade.
+    if all:
+        if package_ids:
+            logger.debug(
+                f"Ignore provided {', '.join(sorted(package_ids))} packages and proceed to a full upgrade..."
+            )
+        for manager in ctx.obj.selected_managers(
+            implements_operation=Operations.upgrade_all
+        ):
+            logger.info(f"Upgrade all outdated packages from {manager.id}...")
+            output = manager.upgrade()
+            if output:
+                logger.info(output)
+        ctx.exit()
+
+    # Specific list of package to upgrade has been requested. We need to
+    # validate them before proceeding.
+
+    # For each package, we list the managers they were sourced from.
+    package_sources = {}
+    for manager in ctx.obj.selected_managers(implements_operation=Operations.upgrade):
+        for package in manager.installed:
+            if package.id in package_ids:
+                package_sources.setdefault(package.id, set()).add(manager.id)
+    logger.debug(f"Managers sourcing each package: {package_sources}")
+
+    for package_id in package_ids:
+
+        # Skip unrecognized packages.
+        if package_id not in package_sources:
+            logger.error(
+                f"{package_id} is not recognized by any of the selected manager."
+                f" Skip its upgrade."
+            )
             continue
 
+        # A package to upgrade that was sourced from multiple managers leads to undefined
+        # behavior. What should we do in this case? Upgrade the package with each manager? Only the first one? A random one?
+        # We choose to play it safe and simply report this ambiguous situation to
+        # the user and skip the package.
+        managers = package_sources[package_id]
+        if len(managers) > 1:
+            logger.error(
+                f"{package_id} was sourced from multiple managers: {', '.join(sorted(managers))}."
+                " Skip its upgrade."
+            )
+            continue
+
+        assert len(managers) == 1
+        manager_id = managers.pop()
+        manager = pool.get(manager_id)
+        logger.info(f"Proceed to upgrade {package_id} with {manager_id}...")
+        output = manager.upgrade(package_id)
         if output:
             logger.info(output)
 
 
-@mpm.command(short_help="Remove a package.", section=MAINTENANCE)
+@mpm.command(aliases=["uninstall"], short_help="Remove a package.", section=MAINTENANCE)
 @argument("package_id", type=STRING, required=True)
 @pass_context
 def remove(ctx, package_id):
-    """Remove the provided package using one of the selected package manager."""
+    """Remove the provided package using one of the selected manager."""
     # Cast generator to tuple because of reuse.
-    selected_managers = tuple(ctx.obj.selected_managers)
+    selected_managers = tuple(
+        ctx.obj.selected_managers(implements_operation=Operations.remove)
+    )
 
     logger.info(
         f"Package manager order: {', '.join([m.id for m in selected_managers])}"
@@ -726,7 +867,7 @@ def remove(ctx, package_id):
                 continue
             logger.info(f"{package_id} has been installed by {manager.id}.")
 
-        # Allow remove subcommand to fail to have the oportunity to catch the CLIError exception and print
+        # Allow remove subcommand to fail to have the opportunity to catch the CLIError exception and print
         # a comprehensive message.
         with patch.object(manager, "stop_on_error", True):
             try:
@@ -740,97 +881,188 @@ def remove(ctx, package_id):
                 continue
 
         echo(output)
-        return
+        ctx.exit()
 
 
 @mpm.command(short_help="Sync local package info.", section=MAINTENANCE)
 @pass_context
 def sync(ctx):
     """Sync local package metadata and info from external sources."""
-    for manager in ctx.obj.selected_managers:
+    for manager in ctx.obj.selected_managers(implements_operation=Operations.sync):
         logger.info(f"Sync {manager.id} package info...")
-        try:
-            manager.sync()
-        except NotImplementedError:
-            logger.warning(f"{manager.id} does not implement sync operation.")
+        manager.sync()
 
 
 @mpm.command(short_help="Cleanup local data.", section=MAINTENANCE)
 @pass_context
 def cleanup(ctx):
     """Cleanup local data, temporary artifacts and removes orphaned dependencies."""
-    for manager in ctx.obj.selected_managers:
+    for manager in ctx.obj.selected_managers(implements_operation=Operations.cleanup):
         logger.info(f"Cleanup {manager.id}...")
-        try:
-            manager.cleanup()
-        except NotImplementedError:
-            logger.warning(f"{manager.id} does not implement cleanup operation.")
+        manager.cleanup()
 
 
-@mpm.command(short_help="Save installed packages to a TOML file.", section=SNAPSHOTS)
-@argument("toml_output", type=File("w"), default="-")
+@mpm.command(
+    aliases=["lock", "freeze", "snapshot"],
+    short_help="Save installed packages to a TOML file.",
+    section=SNAPSHOTS,
+)
+@option(
+    "--overwrite",
+    "--force",
+    "--replace",
+    is_flag=True,
+    default=False,
+    help="Allow the provided TOML file to be silently wiped out if it already exists.",
+)
+@option(
+    "--merge",
+    is_flag=True,
+    default=False,
+    help="Read the provided TOML file and update each entry with the version currently installed on the system. Requires the [TOML_PATH] argument.",
+)
+@option(
+    "--update-version",
+    is_flag=True,
+    default=False,
+    help="Read the provided TOML file and update each existing entry with the version currently installed on the system. Requires the [TOML_PATH] argument.",
+)
+@argument(
+    "toml_path",
+    type=ClickPath(
+        dir_okay=False,
+        writable=True,
+        resolve_path=True,
+        allow_dash=True,
+        path_type=Path,
+    ),
+    default="-",
+)
 @pass_context
-def backup(ctx, toml_output):
+def backup(ctx, overwrite, merge, update_version, toml_path):
     """Dump the list of installed packages to a TOML file.
 
     By default the generated TOML content is displayed directly in the console
     output. So `mpm backup` is the same as a call to `mpm backup -`. To have
     the result written in a file on disk, specify the output file like so:
-    `mpm backup ./mpm-packages.toml`.
+    `mpm backup packages.toml`.
 
-    The TOML file can then be safely consumed by the `mpm restore` subcommand.
+    Files produced by this subcommand can be safely consumed by `mpm restore`.
     """
-    is_stdout = isinstance(toml_output, TextIOWrapper)
-    toml_filepath = toml_output.name if is_stdout else Path(toml_output.name).resolve()
-    logger.info(f"Backup package list to {toml_filepath}")
 
-    if not is_stdout:
-        if toml_filepath.exists() and not toml_filepath.is_file():
-            logger.error("Target file exist and is not a file.")
-            return
-        if toml_filepath.suffix.lower() != ".toml":
-            logger.error("Target file is not a TOML file.")
-            return
+    def is_stdout(filepath):
+        return str(filepath) == "-"
 
-    # Leave some metadata as comment.
-    write = toml_output.write
-    write(f"# Generated by mpm v{__version__}.\n")
-    write(f"# Timestamp: {datetime.now().isoformat()}.\n")
+    if is_stdout(toml_path):
+        if merge:
+            logger.fatal(
+                "--merge requires the [TOML_PATH] argument to point to a file."
+            )
+            ctx.exit(2)
+        if update_version:
+            logger.fatal(
+                "--update-version requires the [TOML_PATH] argument to point to a file."
+            )
+            ctx.exit(2)
+        if overwrite:
+            logger.warning("Ignore the --overwrite/--force/--replace option.")
+        logger.info(f"Print installed package list to {sys.stdout.name}")
+
+    else:
+        if merge and update_version:
+            logger.fatal("--merge and --update-version are mutually exclusive.")
+            ctx.exit(2)
+
+        if merge:
+            logger.info(f"Merge all installed packages into {toml_path}")
+        elif update_version:
+            logger.info(
+                f"Update in-place all versions of installed packages found in {toml_path}"
+            )
+        else:
+            logger.info(f"Dump all installed packages into {toml_path}")
+
+        if toml_path.exists():
+            if overwrite:
+                logger.warning("Target file exist and will be overwritten.")
+            else:
+                if merge or update_version:
+                    logger.warning("Ignore the --overwrite/--force/--replace option.")
+                else:
+                    logger.fatal("Target file exist and will be overwritten.")
+                    ctx.exit(2)
+        elif merge:
+            logger.fatal("--merge requires an existing file.")
+            ctx.exit(2)
+        elif update_version:
+            logger.fatal("--update-version requires an existing file.")
+            ctx.exit(2)
+
+        if toml_path.suffix.lower() != ".toml":
+            logger.fatal("Target file is not a TOML file.")
+            ctx.exit(2)
 
     installed_data = {}
+    fields = (
+        "id",
+        "installed_version",
+    )
+    if merge or update_version:
+        installed_data = tomllib.loads(toml_path.read_text())
 
-    # Create one section for each package manager.
-    for manager in ctx.obj.selected_managers:
-        logger.info(f"Dumping packages from {manager.id}...")
+    @contextmanager
+    def file_writer(filepath):
+        """A context-aware file writer which default to stdout if no path is
+        provided."""
+        if is_stdout(filepath):
+            yield sys.stdout
+        else:
+            writer = filepath.open("w")
+            yield writer
+            writer.close()
 
-        try:
-            packages = tuple(map(dataclasses.asdict, manager.installed))
-        except NotImplementedError:
-            logger.warning(f"{manager.id} does not implement installed operation.")
-            continue
+    with file_writer(toml_path) as f:
 
-        # Prepare data for stats.
-        installed_data[manager.id] = {
-            "id": manager.id,
-            "packages": packages,
-        }
+        # Leave some metadata as comment.
+        f.write(f"# Generated by mpm v{__version__}.\n")
+        f.write(f"# Timestamp: {datetime.now().isoformat()}.\n")
 
-        if packages:
-            write("\n")
-            write(
-                tomli_w.dumps(
-                    {
-                        manager.id: dict(
-                            sorted(
-                                (p["id"], str(p["installed_version"])) for p in packages
-                            )
+        # Create one section for each manager.
+        for manager in ctx.obj.selected_managers(
+            implements_operation=Operations.installed
+        ):
+            logger.info(f"Dumping packages from {manager.id}...")
+
+            packages = tuple(packages_asdict(manager.installed, fields))
+
+            for pkg in packages:
+                # Only update version in that mode if the package is already referenced
+                # into original TOML file.
+                if update_version:
+                    if pkg["id"] in installed_data.get(manager.id, {}):
+                        installed_data[manager.id][pkg["id"]] = str(
+                            pkg["installed_version"]
                         )
-                    }
+                # Insert installed package in data structure for standard dump and merge
+                # mode.
+                else:
+                    installed_data.setdefault(manager.id, {})[pkg["id"]] = str(
+                        pkg["installed_version"]
+                    )
+
+            # Re-sort package list.
+            if installed_data.get(manager.id):
+                installed_data[manager.id] = dict(
+                    sorted(installed_data[manager.id].items())
                 )
-            )
+
+        # Write each section separated by an empty line for readability.
+        for manager_id, packages in installed_data.items():
+            f.write("\n")
+            f.write(tomli_w.dumps({manager_id: packages}))
 
     if ctx.obj.stats:
-        print_stats(installed_data)
+        print_stats(Counter({k: len(v) for k, v in installed_data.items()}))
 
 
 @mpm.command(
@@ -861,13 +1093,15 @@ def restore(ctx, toml_files):
             sections = ", ".join(ignored_sections)
             logger.info(f"Ignore {sections} section{plural}.")
 
-        for manager in ctx.obj.selected_managers:
+        for manager in ctx.obj.selected_managers(
+            implements_operation=Operations.install
+        ):
             if manager.id not in doc:
                 logger.warning(f"No [{manager.id}] section found.")
                 continue
             logger.info(f"Restore {manager.id} packages...")
             for package_id, version in doc[manager.id].items():
-                # Let the command fail if the package manager doesn't implement the
+                # Let the command fail if the manager doesn't implement the
                 # install operation.
                 logger.info(f"Install {package_id} package from {manager.id}...")
                 output = manager.install(package_id)
