@@ -15,13 +15,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import json
 import re
+from typing import Iterator
 
 from click_extra.platform import LINUX, MACOS, WINDOWS
 
-from .. import logger
 from ..base import Package, PackageManager
+from ..capabilities import search_capabilities
 
 
 class Composer(PackageManager):
@@ -45,7 +48,7 @@ class Composer(PackageManager):
     """
 
     @property
-    def installed(self):
+    def installed(self) -> Iterator[Package]:
         """Fetch installed packages.
 
         .. code-block:: shell-session
@@ -80,10 +83,10 @@ class Composer(PackageManager):
             package_list = json.loads(output)
             for package in package_list["installed"]:
                 package_id = package["name"]
-                yield Package(id=package_id, installed_version=package["version"])
+                yield self.package(id=package_id, installed_version=package["version"])
 
     @property
-    def outdated(self):
+    def outdated(self) -> Iterator[Package]:
         """Fetch outdated packages.
 
         .. code-block:: shell-session
@@ -114,13 +117,14 @@ class Composer(PackageManager):
             package_list = json.loads(output)
             for package in package_list["installed"]:
                 package_id = package["name"]
-                yield Package(
+                yield self.package(
                     id=package_id,
                     installed_version=package["version"],
                     latest_version=package["latest"],
                 )
 
-    def search(self, query, extended, exact):
+    @search_capabilities(exact_support=False)
+    def search(self, query: str, extended: bool, exact: bool) -> Iterator[Package]:
         """Fetch matching packages.
 
         .. caution::
@@ -162,9 +166,6 @@ class Composer(PackageManager):
             ► search global --only-name pythonphp/pythonphp
             pythonphp/pythonphp
         """
-        if exact:
-            logger.warning(f"{self.id} does not implement exact search operation.")
-
         search_args = []
         if not extended:
             search_args.append("--only-name")
@@ -180,9 +181,9 @@ class Composer(PackageManager):
         )
 
         for package_id, description in regexp.findall(output):
-            yield Package(id=package_id, description=description)
+            yield self.package(id=package_id, description=description)
 
-    def install(self, package_id):
+    def install(self, package_id: str) -> str:
         """Install one package.
 
         .. code-block:: shell-session
@@ -191,13 +192,19 @@ class Composer(PackageManager):
         """
         return self.run_cli("install", package_id)
 
-    def upgrade_cli(self, package_id=None):
+    def upgrade_all_cli(self) -> tuple[str, ...]:
         """Generates the CLI to upgrade all packages (default) or only the one provided
         as parameter.
 
         .. code-block:: shell-session
 
             ► composer global update
+        """
+        return self.build_cli("update")
+
+    def upgrade_one_cli(self, package_id: str) -> tuple[str, ...]:
+        """Generates the CLI to upgrade all packages (default) or only the one provided
+        as parameter.
 
         .. code-block:: shell-session
 
@@ -205,7 +212,7 @@ class Composer(PackageManager):
         """
         return self.build_cli("update", package_id)
 
-    def cleanup(self):
+    def cleanup(self) -> None:
         """Removes things we don't need anymore.
 
         See: https://getcomposer.org/doc/03-cli.md#clear-cache-clearcache-cc
