@@ -15,12 +15,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import re
+from typing import Iterator
 
 from click_extra.platform import LINUX
 
-from .. import logger
 from ..base import Package, PackageManager
+from ..capabilities import search_capabilities
 
 
 class DNF(PackageManager):
@@ -50,7 +53,7 @@ class DNF(PackageManager):
     list_cmd_regexp = re.compile(r"(\S+)\.\S+\s+(\S+)\s+\S+")
 
     @property
-    def installed(self):
+    def installed(self) -> Iterator[Package]:
         """Fetch installed packages.
 
         .. code-block:: shell-session
@@ -68,10 +71,10 @@ class DNF(PackageManager):
             match = self.list_cmd_regexp.match(package)
             if match:
                 package_id, installed_version = match.groups()
-                yield Package(id=package_id, installed_version=installed_version)
+                yield self.package(id=package_id, installed_version=installed_version)
 
     @property
-    def outdated(self):
+    def outdated(self) -> Iterator[Package]:
         """Fetch outdated packages.
 
         .. code-block:: shell-session
@@ -90,9 +93,10 @@ class DNF(PackageManager):
             match = self.list_cmd_regexp.match(package)
             if match:
                 package_id, latest_version = match.groups()
-                yield Package(id=package_id, latest_version=latest_version)
+                yield self.package(id=package_id, latest_version=latest_version)
 
-    def search(self, query, extended, exact):
+    @search_capabilities(extended_support=False, exact_support=False)
+    def search(self, query: str, extended: bool, exact: bool) -> Iterator[Package]:
         """Fetch matching packages.
 
         .. caution::
@@ -112,11 +116,6 @@ class DNF(PackageManager):
             lvm2-dbusd.noarch : LVM2 D-Bus daemon
             usd-libs.aarch64 : Universal Scene Description library
         """
-        if extended:
-            logger.warning(f"{self.id} does not implement extended search operation.")
-        if exact:
-            logger.warning(f"{self.id} does not implement exact search operation.")
-
         output = self.run_cli("search", query)
 
         regexp = re.compile(r"(\S+)\.\S+\s:\s(\S+)")
@@ -130,9 +129,9 @@ class DNF(PackageManager):
             match = regexp.match(line)
             if match:
                 package_id, description = match.groups()
-                yield Package(id=package_id, description=description)
+                yield self.package(id=package_id, description=description)
 
-    def install(self, package_id):
+    def install(self, package_id: str) -> str:
         """Install one package.
 
         .. code-block:: shell-session
@@ -141,13 +140,19 @@ class DNF(PackageManager):
         """
         return self.run_cli("--assumeyes", "install", package_id, sudo=True)
 
-    def upgrade_cli(self, package_id=None):
+    def upgrade_all_cli(self) -> tuple[str, ...]:
         """Generates the CLI to upgrade all packages (default) or only the one provided
         as parameter.
 
         .. code-block:: shell-session
 
             ► sudo dnf --color=never --assumeyes upgrade
+        """
+        return self.build_cli("upgrade", sudo=True)
+
+    def upgrade_one_cli(self, package_id: str) -> tuple[str, ...]:
+        """Generates the CLI to upgrade all packages (default) or only the one provided
+        as parameter.
 
         .. code-block:: shell-session
 
@@ -155,7 +160,7 @@ class DNF(PackageManager):
         """
         return self.build_cli("upgrade", package_id, sudo=True)
 
-    def sync(self):
+    def sync(self) -> None:
         """Sync package metadata.
 
         .. code-block:: shell-session
@@ -164,7 +169,7 @@ class DNF(PackageManager):
         """
         self.run_cli("check-update")
 
-    def cleanup(self):
+    def cleanup(self) -> None:
         """Removes things we don't need anymore.
 
         .. code-block:: shell-session
