@@ -15,17 +15,34 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import re
+from typing import Iterator
 
 from click_extra.platform import LINUX, MACOS, WINDOWS
 
-from .. import logger
 from ..base import Package, PackageManager
+from ..capabilities import search_capabilities
 from ..version import parse_version
 
 
 class Gem(PackageManager):
 
+    """
+    ..tip::
+
+        Installs require ``sudo`` on system ruby. I (@tresni) recommend doing something like:
+
+        .. code-block:: shell-session
+
+            ► sudo dseditgroup -o edit -a -t user wheel
+
+        And then do ``visudo`` to make it so the ``wheel`` group does not require
+        a password. There is a line already there for it, you just need to
+        uncomment it and save.
+    """
+
     name = "Ruby Gems"
 
     homepage_url = "https://rubygems.org"
@@ -49,7 +66,7 @@ class Gem(PackageManager):
     post_args = ("--quiet",)  # Silence command progress meter
 
     @property
-    def installed(self):
+    def installed(self) -> Iterator[Package]:
         """Fetch installed packages.
 
         .. code-block:: shell-session
@@ -85,10 +102,10 @@ class Gem(PackageManager):
                     for v in re.compile(r",|default:| ").split(versions)
                     if v
                 )
-                yield Package(id=package_id, installed_version=version)
+                yield self.package(id=package_id, installed_version=version)
 
     @property
-    def outdated(self):
+    def outdated(self) -> Iterator[Package]:
         """Fetch outdated packages.
 
         .. code-block:: shell-session
@@ -108,13 +125,14 @@ class Gem(PackageManager):
             match = regexp.match(package)
             if match:
                 package_id, current_version, latest_version = match.groups()
-                yield Package(
+                yield self.package(
                     id=package_id,
                     installed_version=current_version,
                     latest_version=latest_version,
                 )
 
-    def search(self, query, extended, exact):
+    @search_capabilities(extended_support=False)
+    def search(self, query: str, extended: bool, exact: bool) -> Iterator[Package]:
         """Fetch matching packages.
 
         .. caution::
@@ -138,9 +156,6 @@ class Gem(PackageManager):
             ► gem search python --versions --exact --quiet
             python (0.0.1)
         """
-        if extended:
-            logger.warning(f"{self.id} does not implement extended search operation.")
-
         search_arg = []
         if exact:
             search_arg.append("--exact")
@@ -160,9 +175,9 @@ class Gem(PackageManager):
         )
 
         for package_id, version in regexp.findall(output):
-            yield Package(id=package_id, latest_version=version)
+            yield self.package(id=package_id, latest_version=version)
 
-    def install(self, package_id):
+    def install(self, package_id: str) -> str:
         """Install one package.
 
         .. code-block:: shell-session
@@ -192,27 +207,28 @@ class Gem(PackageManager):
             auto_post_args=False,
         )
 
-    def upgrade_cli(self, package_id=None):
+    def upgrade_all_cli(self) -> tuple[str, ...]:
         """Generates the CLI to upgrade all packages (default) or only the one provided
         as parameter.
 
         .. code-block:: shell-session
 
             ► gem update --user-install --quiet
+        """
+        return self.build_cli(
+            "update",
+            "--user-install",
+            *self.post_args,
+            auto_post_args=False,
+        )
+
+    def upgrade_one_cli(self, package_id: str) -> tuple[str, ...]:
+        """Generates the CLI to upgrade all packages (default) or only the one provided
+        as parameter.
 
         .. code-block:: shell-session
 
             ► gem update --user-install --quiet markdown
-
-        Installs require ``sudo`` on system ruby. I (@tresni) recommend doing something like:
-
-        .. code-block:: shell-session
-
-            ► sudo dseditgroup -o edit -a -t user wheel
-
-        And then do ``visudo`` to make it so the ``wheel`` group does not require
-        a password. There is a line already there for it, you just need to
-        uncomment it and save.
         """
         return self.build_cli(
             "update",
@@ -222,7 +238,7 @@ class Gem(PackageManager):
             auto_post_args=False,
         )
 
-    def cleanup(self):
+    def cleanup(self) -> None:
         """Removes things we don't need anymore.
 
         .. code-block:: shell-session
