@@ -15,12 +15,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import json
 import re
 from operator import methodcaller
+from typing import Iterator
 
 from click_extra.platform import LINUX, MACOS
 
+from .. import logger
 from ..base import Package, PackageManager
 from ..version import parse_version
 
@@ -39,7 +43,7 @@ class Homebrew(PackageManager):
     requirement = "2.7.0"
 
     # Declare this manager as virtual, i.e. not tied to a real CLI.
-    cli_names = None
+    virtual = True
 
     extra_env = {
         # Disable analytics.
@@ -65,7 +69,7 @@ class Homebrew(PackageManager):
     """
 
     @property
-    def installed(self):
+    def installed(self) -> Iterator[Package]:
         """Fetch installed packages.
 
         .. code-block:: shell-session
@@ -121,10 +125,10 @@ class Homebrew(PackageManager):
         ):
             # Keep highest version found.
             version = max(map(parse_version, versions.split()))
-            yield Package(id=package_id, installed_version=version)
+            yield self.package(id=package_id, installed_version=version)
 
     @property
-    def outdated(self):
+    def outdated(self) -> Iterator[Package]:
         """Fetch outdated packages.
 
         .. code-block:: shell-session
@@ -200,9 +204,9 @@ class Homebrew(PackageManager):
                   "current_version": "4.0.3"
                 },
                 {
-                  "name": "qlcolorcode",
-                  "installed_versions": "3.0.2",
-                  "current_version": "3.1.1"
+                  "name": "lg-onscreen-control",
+                  "installed_versions": "5.33,cV8xqv5TSZA.upgrading, 5.47,yi5XuIZw6hg",
+                  "current_version": "5.48,uYXSwyUCNFBbSch9PFw"
                 }
               ]
             }
@@ -220,22 +224,29 @@ class Homebrew(PackageManager):
             package_list = json.loads(output)
             for pkg_info in package_list["formulae"] + package_list["casks"]:
 
-                version = pkg_info["installed_versions"]
-                if not isinstance(version, str):
-                    version = max(map(parse_version, version))
-                latest_version = pkg_info["current_version"]
+                # Interpret installed versions.
+                versions = pkg_info["installed_versions"]
+                if isinstance(versions, str):
+                    versions = versions.split(", ")
+                installed_version = max(map(parse_version, versions))
 
-                # Skip packages in undetermined state.
-                if version == latest_version:
+                latest_version = parse_version(pkg_info["current_version"])
+
+                # Skip packages not offering upgradeable version.
+                package_id = pkg_info["name"]
+                if installed_version == latest_version:
+                    logger.debug(
+                        f"Ignore {package_id} upgrade from {installed_version} to {latest_version}."
+                    )
                     continue
 
-                yield Package(
-                    id=pkg_info["name"],
-                    installed_version=version,
+                yield self.package(
+                    id=package_id,
+                    installed_version=installed_version,
                     latest_version=latest_version,
                 )
 
-    def search(self, query, extended, exact):
+    def search(self, query: str, extended: bool, exact: bool) -> Iterator[Package]:
         """Fetch matching packages.
 
         .. caution::
@@ -329,7 +340,7 @@ class Homebrew(PackageManager):
 
             for package_id, _, package_name, description in regexp.findall(output):
                 matched_ids.add(package_id)
-                pkg = Package(id=package_id, name=package_name)
+                pkg = self.package(id=package_id, name=package_name)
                 if description != "[no description]":
                     pkg.description = description
                 yield pkg
@@ -351,9 +362,9 @@ class Homebrew(PackageManager):
         for package_id in regexp.findall(output):
             # Deduplicate search results.
             if package_id not in matched_ids:
-                yield Package(id=package_id)
+                yield self.package(id=package_id)
 
-    def install(self, package_id):
+    def install(self, package_id: str) -> str:
         """Install one package.
 
         .. code-block:: shell-session
@@ -378,7 +389,7 @@ class Homebrew(PackageManager):
         """
         return self.run_cli("install", package_id)
 
-    def upgrade_cli(self, package_id=None):
+    def upgrade_all_cli(self) -> tuple[str, ...]:
         """Generates the CLI to upgrade all packages (default) or only the one provided
         as parameter.
 
@@ -430,6 +441,14 @@ class Homebrew(PackageManager):
             ==> Moving Screen Saver to '/Users/kde/Library/Screen Savers/Aerial.saver'.
             ==> Purging files for version 2.0.7 of Cask aerial
             🍺  aerial was successfully upgraded!
+        """
+        return self.build_cli("upgrade")
+
+    def upgrade_one_cli(self, package_id: str) -> tuple[str, ...]:
+        """Generates the CLI to upgrade all packages (default) or only the one provided
+        as parameter.
+
+        ``brew`` and ``cask`` share the same command.
 
         .. code-block:: shell-session
 
@@ -448,7 +467,17 @@ class Homebrew(PackageManager):
         """
         return self.build_cli("upgrade", package_id)
 
-    def sync(self):
+    def remove(self, package_id: str) -> str:
+        """Removes a package.
+
+        .. code-block:: shell-session
+
+            ► brew uninstall bat
+            Uninstalling /usr/local/Cellar/bat/0.21.0... (14 files, 5MB)
+        """
+        return self.run_cli("uninstall", package_id)
+
+    def sync(self) -> None:
         """Sync package metadata.
 
         .. code-block:: shell-session
@@ -458,7 +487,7 @@ class Homebrew(PackageManager):
         """
         self.run_cli("update", "--quiet", auto_post_args=False)
 
-    def cleanup(self):
+    def cleanup(self) -> None:
         """Removes things we don't need anymore.
 
         Scrub the cache, including latest version's downloads. Also remove unused
