@@ -15,12 +15,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import re
+from typing import Iterator
 
 from click_extra.platform import MACOS
 
-from .. import logger
 from ..base import Package, PackageManager
+from ..capabilities import search_capabilities
 
 
 class MAS(PackageManager):
@@ -44,7 +47,7 @@ class MAS(PackageManager):
     """
 
     @property
-    def installed(self):
+    def installed(self) -> Iterator[Package]:
         """Fetch installed packages.
 
         .. code-block:: shell-session
@@ -72,10 +75,12 @@ class MAS(PackageManager):
         )
 
         for package_id, package_name, version in regexp.findall(output):
-            yield Package(id=package_id, name=package_name, installed_version=version)
+            yield self.package(
+                id=package_id, name=package_name, installed_version=version
+            )
 
     @property
-    def outdated(self):
+    def outdated(self) -> Iterator[Package]:
         """Fetch outdated packages.
 
         .. code-block:: shell-session
@@ -107,14 +112,15 @@ class MAS(PackageManager):
             installed_version,
             latest_version,
         ) in regexp.findall(output):
-            yield Package(
+            yield self.package(
                 id=package_id,
                 name=package_name,
                 installed_version=installed_version,
                 latest_version=latest_version,
             )
 
-    def search(self, query, extended, exact):
+    @search_capabilities(extended_support=False, exact_support=False)
+    def search(self, query: str, extended: bool, exact: bool) -> Iterator[Package]:
         """Fetch matching packages.
 
         .. caution::
@@ -130,11 +136,6 @@ class MAS(PackageManager):
               1164498373  PythonGames     (1.0)
               1400050251  Pythonic        (1.0.0)
         """
-        if extended:
-            logger.warning(f"{self.id} does not implement extended search operation.")
-        if exact:
-            logger.warning(f"{self.id} does not implement exact search operation.")
-
         output = self.run_cli("search", query)
 
         regexp = re.compile(
@@ -151,9 +152,9 @@ class MAS(PackageManager):
         )
 
         for package_id, package_name, version in regexp.findall(output):
-            yield Package(id=package_id, name=package_name, latest_version=version)
+            yield self.package(id=package_id, name=package_name, latest_version=version)
 
-    def install(self, package_id):
+    def install(self, package_id: str) -> str:
         """Install one package.
 
         .. code-block:: shell-session
@@ -162,13 +163,19 @@ class MAS(PackageManager):
         """
         return self.run_cli("install", package_id)
 
-    def upgrade_cli(self, package_id=None):
+    def upgrade_all_cli(self) -> tuple[str, ...]:
         """Generates the CLI to upgrade all packages (default) or only the one provided
         as parameter.
 
         .. code-block:: shell-session
 
             ► mas upgrade
+        """
+        return self.build_cli("upgrade")
+
+    def upgrade_one_cli(self, package_id: str) -> tuple[str, ...]:
+        """Generates the CLI to upgrade all packages (default) or only the one provided
+        as parameter.
 
         .. code-block:: shell-session
 
