@@ -15,13 +15,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import json
+from typing import Any, Iterator
 
 from boltons.iterutils import remap
 from click_extra.platform import LINUX, MACOS, WINDOWS
 
-from .. import logger
-from ..base import Package, PackageManager
+from ..base import Package, PackageManager, _Arg, _NestedArgs
+from ..capabilities import search_capabilities
 
 
 class NPM(PackageManager):
@@ -35,9 +38,19 @@ class NPM(PackageManager):
     requirement = "4.0.0"
 
     pre_args = (
+        # Operates in "global" mode, so that packages are installed into the
+        # prefix folder instead of the current working directory.
         "--global",
+        # Suppress the progress bar.
         "--progress=false",
+        # Suppress the update notification when using an older version of npm than
+        # the latest.
         "--no-update-notifier",
+        # Hide the message displayed at the end of each install that acknowledges
+        # the number of dependencies looking for funding.
+        "--no-fund",
+        # Disable sending of audit reports to the configured registries.
+        "--no-audit",
     )
 
     """
@@ -47,14 +60,14 @@ class NPM(PackageManager):
         6.13.7
     """
 
-    def run_cli(self, *args, **kwargs):
+    def run_cli(self, *args: _Arg | _NestedArgs, **kwargs: Any) -> str:
         """Like the common run_cli helper, but silence NPM's JSON output on error.
 
         NPM is prone to breakage if local node version is not in sync:
 
         .. code-block:: shell-session
 
-            ► npm --global --progress=false --no-update-notifier --json outdated
+            ► npm --global --progress=false --no-update-notifier --no-fund --no-audit --json outdated
             {
               "error": {
                 "code": "ERR_OUT_OF_RANGE",
@@ -71,17 +84,17 @@ class NPM(PackageManager):
         # (unless the --stop-on-error option is provided).
         if "--json" in args:
             if output and self.cli_errors:
-                output = None
+                output = ""
 
         return output
 
     @property
-    def installed(self):
+    def installed(self) -> Iterator[Package]:
         """Fetch installed packages.
 
         .. code-block:: shell-session
 
-            ► npm --global --progress=false --no-update-notifier --json list | jq
+            ► npm --global --progress=false --no-update-notifier --no-fund --no-audit --json list | jq
             {
               "dependencies": {
                 "npm": {
@@ -122,23 +135,25 @@ class NPM(PackageManager):
 
         if output:
 
-            def visit(path, key, value):
+            def visit(path, key: str, value: str) -> bool:
                 if key == "version":
                     package_id = path[-1]
-                    installed.append(Package(id=package_id, installed_version=value))
+                    installed.append(
+                        self.package(id=package_id, installed_version=value)
+                    )
                 return True
 
             remap(json.loads(output), visit=visit)
 
-        return installed
+        yield from installed
 
     @property
-    def outdated(self):
+    def outdated(self) -> Iterator[Package]:
         """Fetch outdated packages.
 
         .. code-block:: shell-session
 
-            ► npm --global --progress=false --no-update-notifier --json outdated
+            ► npm --global --progress=false --no-update-notifier --no-fund --no-audit --json outdated | jq
             {
               "my-linked-package": {
                 "current": "0.0.0-development",
@@ -150,7 +165,7 @@ class NPM(PackageManager):
                 "current": "3.10.3",
                 "wanted": "3.10.5",
                 "latest": "3.10.5",
-                "location": "/Users/..."
+                "location": "/opt/homebrew/lib/node_modules/npm"
               }
             }
         """
@@ -160,13 +175,15 @@ class NPM(PackageManager):
             for package_id, values in json.loads(output).items():
                 if values["wanted"] == "linked":
                     continue
-                yield Package(
-                    id=f"{package_id}@{values['latest']}",
-                    installed_version=values["current"],
+                yield self.package(
+                    id=f"{package_id}",
+                    # It seems "current" is not always populated.
+                    installed_version=values.get("current"),
                     latest_version=values["latest"],
                 )
 
-    def search(self, query, extended, exact):
+    @search_capabilities(exact_support=False)
+    def search(self, query: str, extended: bool, exact: bool) -> Iterator[Package]:
         """Fetch matching packages.
 
         Doc: https://docs.npmjs.com/cli/search.html
@@ -176,7 +193,7 @@ class NPM(PackageManager):
 
         .. code-block:: shell-session
 
-            ► npm --global --progress=false --no-update-notifier search --json python | jq
+            ► npm --global --progress=false --no-update-notifier --no-fund --no-audit search --json python | jq
             [
               {
                 "name": "python",
@@ -244,11 +261,8 @@ class NPM(PackageManager):
 
         .. code-block:: shell-session
 
-            ► npm --global --progress=false --no-update-notifier search --json --no-description python | jq
+            ► npm --global --progress=false --no-update-notifier --no-fund --no-audit search --json --no-description python | jq
         """
-        if exact:
-            logger.warning(f"{self.id} does not implement exact search operation.")
-
         search_args = []
         if not extended:
             search_args.append("--no-description")
@@ -257,38 +271,47 @@ class NPM(PackageManager):
 
         if output:
             for pkg in json.loads(output):
-                yield Package(
+                yield self.package(
                     id=pkg.get("name"),
                     description=pkg.get("description"),
                     latest_version=pkg.get("version"),
                 )
 
-    def install(self, package_id):
+    def install(self, package_id: str) -> str:
         """Install one package.
 
         .. code-block:: shell-session
 
-            ► npm --global --progress=false --no-update-notifier install raven
-        """
-        return self.run_cli("install", package_id)
+            ► npm --global --progress=false --no-update-notifier --no-fund --no-audit install markdown
 
-    def upgrade_cli(self, package_id=None, version=None):
+            added 3 packages in 3s
+        """
+        return self.run_cli("install", "--no-fund", "--no-audit", package_id)
+
+    def upgrade_all_cli(self) -> tuple[str, ...]:
         """Generates the CLI to upgrade all packages (default) or only the one provided
         as parameter.
 
         .. code-block:: shell-session
 
-            ► npm --global --progress=false --no-update-notifier update
+            ► npm --global --progress=false --no-update-notifier --no-fund --no-audit upgrade
+        """
+        return self.build_cli("update")
+
+    def upgrade_one_cli(self, package_id) -> tuple[str, ...]:
+        """Generates the CLI to upgrade the package provided as parameter.
 
         .. code-block:: shell-session
 
-            ► npm --global --progress=false --no-update-notifier install raven
+            ► npm --global --progress=false --no-update-notifier --no-fund --no-audit upgrade raven
         """
-        cmd_args = ("update",)
-        if package_id:
-            cmd_args = (
-                "install",
-                f"{package_id}@{version}" if version else package_id,
-            )
+        return self.build_cli("upgrade", f"{package_id}")
 
-        return self.build_cli(*cmd_args)
+    def remove(self, package_id: str) -> str:
+        """Remove one package and one only.
+
+        .. code-block:: shell-session
+
+            ► npm --global --progress=false --no-update-notifier --no-fund --no-audit uninstall raven
+        """
+        return self.run_cli("uninstall", package_id)
