@@ -15,12 +15,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import re
+from typing import Iterator
 
 from click_extra.platform import LINUX
 
-from .. import logger
 from ..base import Package, PackageManager
+from ..capabilities import search_capabilities
 
 
 class OPKG(PackageManager):
@@ -40,7 +43,7 @@ class OPKG(PackageManager):
     """
 
     @property
-    def installed(self):
+    def installed(self) -> Iterator[Package]:
         """Fetch installed packages.
 
         .. code-block:: shell-session
@@ -74,10 +77,10 @@ class OPKG(PackageManager):
             match = regexp.match(package)
             if match:
                 package_id, installed_version = match.groups()
-                yield Package(id=package_id, installed_version=installed_version)
+                yield self.package(id=package_id, installed_version=installed_version)
 
     @property
-    def outdated(self):
+    def outdated(self) -> Iterator[Package]:
         """Fetch outdated packages.
 
         .. code-block:: shell-session
@@ -93,13 +96,14 @@ class OPKG(PackageManager):
             match = regexp.match(package)
             if match:
                 package_id, installed_version, latest_version = match.groups()
-                yield Package(
+                yield self.package(
                     id=package_id,
                     latest_version=latest_version,
                     installed_version=installed_version,
                 )
 
-    def search(self, query, extended, exact):
+    @search_capabilities(extended_support=False, exact_support=False)
+    def search(self, query: str, extended: bool, exact: bool) -> Iterator[Package]:
         """Fetch matching packages.
 
         .. warning::
@@ -113,11 +117,6 @@ class OPKG(PackageManager):
 
             ► opkg list
         """
-        if extended:
-            logger.warning(f"{self.id} does not implement extended search operation.")
-        if exact:
-            logger.warning(f"{self.id} does not implement exact search operation.")
-
         output = self.run_cli("list")
 
         regexp = re.compile(
@@ -132,11 +131,11 @@ class OPKG(PackageManager):
         )
 
         for package_id, version, description in regexp.findall(output):
-            yield Package(
+            yield self.package(
                 id=package_id, description=description, latest_version=version
             )
 
-    def install(self, package_id):
+    def install(self, package_id: str) -> str:
         """Install one package.
 
         .. code-block:: shell-session
@@ -145,13 +144,19 @@ class OPKG(PackageManager):
         """
         return self.run_cli("install", package_id)
 
-    def upgrade_cli(self, package_id=None):
+    def upgrade_all_cli(self) -> tuple[str, ...]:
         """Generates the CLI to upgrade all packages (default) or only the one provided
         as parameter.
 
         .. code-block:: shell-session
 
             ► opkg upgrade
+        """
+        return self.build_cli("upgrade")
+
+    def upgrade_one_cli(self, package_id: str) -> tuple[str, ...]:
+        """Generates the CLI to upgrade all packages (default) or only the one provided
+        as parameter.
 
         .. code-block:: shell-session
 
@@ -159,7 +164,7 @@ class OPKG(PackageManager):
         """
         return self.build_cli("upgrade", package_id)
 
-    def sync(self):
+    def sync(self) -> None:
         """Sync package metadata.
 
         .. code-block:: shell-session
