@@ -15,12 +15,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import re
+from typing import Iterator
 
 from click_extra.platform import LINUX
 
-from .. import logger
 from ..base import Package, PackageManager
+from ..capabilities import search_capabilities
 
 
 class Pacman(PackageManager):
@@ -50,7 +53,7 @@ class Pacman(PackageManager):
     """
 
     @property
-    def installed(self):
+    def installed(self) -> Iterator[Package]:
         """Fetch installed packages.
 
         .. code-block:: shell-session
@@ -72,10 +75,10 @@ class Pacman(PackageManager):
             match = regexp.match(package)
             if match:
                 package_id, installed_version = match.groups()
-                yield Package(id=package_id, installed_version=installed_version)
+                yield self.package(id=package_id, installed_version=installed_version)
 
     @property
-    def outdated(self):
+    def outdated(self) -> Iterator[Package]:
         """Fetch outdated packages.
 
         .. code-block:: shell-session
@@ -91,13 +94,14 @@ class Pacman(PackageManager):
             match = regexp.match(package)
             if match:
                 package_id, installed_version, latest_version = match.groups()
-                yield Package(
+                yield self.package(
                     id=package_id,
                     latest_version=latest_version,
                     installed_version=installed_version,
                 )
 
-    def search(self, query, extended, exact):
+    @search_capabilities(extended_support=False)
+    def search(self, query: str, extended: bool, exact: bool) -> Iterator[Package]:
         """Fetch matching packages.
 
         .. caution::
@@ -121,27 +125,24 @@ class Pacman(PackageManager):
             extra/firefox-i18n-ast 99.0-1
                 Asturian language pack for Firefox
         """
-        if extended:
-            logger.warning(f"{self.id} does not implement extended search operation.")
-
         if exact:
             query = f"^{query}$"
 
         output = self.run_cli("--sync", "--search", query)
 
         regexp = re.compile(
-            r"(?P<package_id>\S+)\s+(?P<version>\S+).*\n\s+(?P<description>.+)",
+            r"(?P<repo_id>\S+?)/(?P<package_id>\S+)\s+(?P<version>\S+).*\n\s+(?P<description>.+)",
             re.MULTILINE | re.VERBOSE,
         )
 
-        for package_id, version, description in regexp.findall(output):
-            yield Package(
+        for repo_id, package_id, version, description in regexp.findall(output):
+            yield self.package(
                 id=package_id,
                 description=description,
                 latest_version=version,
             )
 
-    def install(self, package_id):
+    def install(self, package_id: str) -> str:
         """Install one package.
 
         .. code-block:: shell-session
@@ -150,33 +151,34 @@ class Pacman(PackageManager):
         """
         return self.run_cli("--sync", package_id, sudo=True)
 
-    def upgrade_cli(self, package_id=None):
+    def upgrade_all_cli(self) -> tuple[str, ...]:
         """Generates the CLI to upgrade the package provided as parameter.
 
         .. code-block:: shell-session
 
             ► sudo pacman --noconfirm --sync --refresh --sysupgrade
+        """
+        return self.build_cli("--sync", "--refresh", "--sysupgrade", sudo=True)
+
+    def upgrade_one_cli(self, package_id: str) -> tuple[str, ...]:
+        """Generates the CLI to upgrade the package provided as parameter.
 
         .. code-block:: shell-session
 
             ► sudo pacman --noconfirm --sync firefox
         """
-        params = ("--refresh", "--sysupgrade")
-        if package_id:
-            params = (package_id,)
+        return self.build_cli("--sync", package_id, sudo=True)
 
-        return self.build_cli("--sync", *params, sudo=True)
-
-    def remove(self, package_id):
+    def remove(self, package_id: str) -> str:
         """Removes a package.
 
         .. code-block:: shell-session
 
             ► sudo pacman --noconfirm --remove firefox
         """
-        self.run_cli("--remove", package_id, sudo=True)
+        return self.run_cli("--remove", package_id, sudo=True)
 
-    def sync(self):
+    def sync(self) -> None:
         """Sync package metadata.
 
         .. code-block:: shell-session
@@ -185,7 +187,7 @@ class Pacman(PackageManager):
         """
         self.run_cli("--sync", "--refresh")
 
-    def cleanup(self):
+    def cleanup(self) -> None:
         """Removes things we don't need anymore.
 
         .. code-block:: shell-session
