@@ -15,13 +15,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import json
 import re
+from typing import Iterator
 
 from click_extra.platform import LINUX, MACOS, WINDOWS
 
-from .. import logger
 from ..base import Package, PackageManager
+from ..capabilities import search_capabilities
 
 
 class Pip(PackageManager):
@@ -33,7 +36,7 @@ class Pip(PackageManager):
     Source: https://snarky.ca/why-you-should-use-python-m-pip/
     """
 
-    homepage_url = "https://pypi.org"
+    homepage_url = "https://pip.pypa.io"
 
     platforms = frozenset({MACOS, LINUX, WINDOWS})
 
@@ -59,7 +62,7 @@ class Pip(PackageManager):
     """
 
     @property
-    def installed(self):
+    def installed(self) -> Iterator[Package]:
         """Fetch installed packages.
 
         .. code-block:: shell-session
@@ -100,11 +103,12 @@ class Pip(PackageManager):
 
         if output:
             for package in json.loads(output):
-                package_id = package["name"]
-                yield Package(id=package_id, installed_version=package["version"])
+                yield self.package(
+                    id=package["name"], installed_version=package["version"]
+                )
 
     @property
-    def outdated(self):
+    def outdated(self) -> Iterator[Package]:
         """Fetch outdated packages.
 
         .. code-block:: shell-session
@@ -174,14 +178,17 @@ class Pip(PackageManager):
 
         if output:
             for package in json.loads(output):
-                package_id = package["name"]
-                yield Package(
-                    id=package_id,
+                yield self.package(
+                    id=package["name"],
                     installed_version=package["version"],
                     latest_version=package["latest_version"],
                 )
 
-    def search_xxx_disabled(self, query, extended, exact):
+    @search_capabilities(extended_support=False, exact_support=False)
+    def search_xxx_disabled(
+        self, query: str, extended: bool, exact: bool
+    ) -> Iterator[Package]:
+
         """Fetch matching packages.
 
         .. warning::
@@ -212,13 +219,6 @@ class Pip(PackageManager):
             collective.js.abcjs (1.10)  - UNKNOWN
             cosmo (1.0.5)               - Python ABC sampler
         """
-        if not extended:
-            logger.warning(
-                f"{self.id} does not implement non-extended search operation."
-            )
-        if exact:
-            logger.warning(f"{self.id} does not implement exact search operation.")
-
         output = self.run_cli("search", query)
 
         regexp = re.compile(
@@ -235,13 +235,13 @@ class Pip(PackageManager):
         )
 
         for package_id, version, description in regexp.findall(output):
-            yield Package(
+            yield self.package(
                 id=package_id,
                 description=description,
                 latest_version=version,
             )
 
-    def install(self, package_id):
+    def install(self, package_id: str) -> str:
         """Install one package.
 
         .. code-block:: shell-session
@@ -257,7 +257,7 @@ class Pip(PackageManager):
         """
         return self.run_cli("install", package_id)
 
-    def upgrade_cli(self, package_id):
+    def upgrade_one_cli(self, package_id: str) -> tuple[str, ...]:
         """Generates the CLI to upgrade the package provided as parameter.
 
         .. code-block:: shell-session
@@ -271,24 +271,21 @@ class Pip(PackageManager):
                 Uninstalling six-1.14.0:
                   Successfully uninstalled six-1.14.0
             Successfully installed six-1.15.0
+
+        ,, note::
+
+            Pip lacks support of a proper full upgrade command. Raising an error let the
+            parent class upgrade packages one by one.
+
+            See: https://github.com/pypa/pip/issues/59
         """
         return self.build_cli("install", "--user", "--upgrade", package_id)
 
-    def upgrade_all_cli(self):
-        """Generates the CLI to upgrade all packages.
-
-        Pip lacks support of a proper full upgrade command. Raising an error let the
-        parent class upgrade packages one by one.
-
-        See: https://github.com/pypa/pip/issues/59
-        """
-        raise NotImplementedError
-
-    def remove(self, package_id):
+    def remove(self, package_id: str) -> str:
         """Remove one package.
 
         .. code-block:: shell-session
 
-            ► python -m pip --no-color uninstall arrow
+            ► python -m pip --no-color uninstall --yes arrow
         """
-        return self.run_cli("uninstall", package_id)
+        return self.run_cli("uninstall", "--yes", package_id)
