@@ -15,6 +15,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
+from typing import Iterator
+
 from click_extra.platform import LINUX, MACOS, WINDOWS
 
 from ..base import Package, PackageManager
@@ -42,7 +46,7 @@ class VSCode(PackageManager):
     """
 
     @property
-    def installed(self):
+    def installed(self) -> Iterator[Package]:
         """Fetch installed packages.
 
         .. code-block:: shell-session
@@ -60,9 +64,9 @@ class VSCode(PackageManager):
 
         for package in output.splitlines():
             package_id, installed_version = package.split("@")
-            yield Package(id=package_id, installed_version=installed_version)
+            yield self.package(id=package_id, installed_version=installed_version)
 
-    def install(self, package_id):
+    def install(self, package_id: str) -> str:
         """Install one package.
 
         .. code-block:: shell-session
