@@ -16,6 +16,8 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 
+from __future__ import annotations
+
 from functools import partial
 
 import pytest
@@ -35,7 +37,8 @@ def invoke(invoke_extra):
 
 @pytest.fixture(scope="class")
 def subcmd():
-    """Fixture used in ``test_cli_*.py`` files to set the sub-command in all CLI calls.
+    """Fixture used in ``test_cli_*.py`` files to set the sub-command arguments in all
+    CLI calls.
 
     Must returns a string or an iterable of strings. Defaults to ``None``, which allows
     tests relying on this fixture to selectively skip running.
