@@ -15,6 +15,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import json
 import re
 import subprocess
@@ -24,7 +26,8 @@ from boltons.strutils import strip_ansi
 from click_extra.tabulate import TabularOutputFormatter
 
 from .. import __version__
-from ..bar_plugin import MPMPlugin
+from ..bar_plugin import MPMPlugin  # type: ignore
+from ..base import Operations
 from ..pool import pool
 
 """ Common tests for all CLI basic features and templates for subcommands. """
@@ -77,9 +80,8 @@ class TestBaseCLI:
         conf_path = create_config("conf.toml", TEST_CONF_FILE)
         result = invoke("--config", str(conf_path), "managers", color=False)
         assert result.exit_code == 0
-        assert " │ pip │ " in result.stdout
-        assert " │ npm │ " in result.stdout
-        assert " │ gem │ " in result.stdout
+        for mid in ("pip", "npm", "gem"):
+            assert re.search(rf"│ {mid}\s+│ ", result.stdout)
         assert " brew " not in result.stdout
         assert " brew cask " not in result.stdout
         assert "debug: " in result.stderr
@@ -95,9 +97,8 @@ class TestBaseCLI:
             color=False,
         )
         assert result.exit_code == 0
-        assert " │ pip │ " in result.stdout
-        assert " │ npm │ " in result.stdout
-        assert " │ gem │ " in result.stdout
+        for mid in ("pip", "npm", "gem"):
+            assert re.search(rf"│ {mid}\s+│ ", result.stdout)
         assert " brew " not in result.stdout
         assert " cask " not in result.stdout
         assert "error: " not in result.stderr
@@ -136,7 +137,7 @@ class CLISubCommandTests:
         matter because:
         * ``<stdout>`` and ``<stderr>`` gets mangled
         * paging is async
-        * we may introduce parrallel execution of manager in the future
+        * we may introduce parallel execution of manager in the future
 
         This explain the use of ``set()`` everywhere.
 
@@ -166,8 +167,7 @@ class CLISubCommandTests:
                 bool(
                     re.search(
                         rf"warning: {mid} does not implement "
-                        r"(installed|outdated|search|install|upgrade|upgrade_all|remove|sync|cleanup) "
-                        "operation.",
+                        rf"Operations\.({'|'.join(Operations.__members__.keys())}).",
                         stderr,
                     )
                 ),
@@ -176,7 +176,7 @@ class CLISubCommandTests:
                 # Match output of managers command.
                 bool(
                     re.search(
-                        rf"\s+│\s+{mid}\s+│\s+(✓|✘).+│\s+(✓|✘)",
+                        rf"│\s+{mid}\s+│.+│\s+(✓|✘).+│\s+(✓|✘)",
                         stdout,
                     )
                 ),
@@ -194,7 +194,14 @@ class CLISubCommandTests:
                     )
                 ),
                 # Upgrade command.
-                f"Updating all outdated packages from {mid}..." in stderr,
+                f"warning: {mid} does not implement upgrade_all_cli." in stderr,
+                f"Upgrade all outdated packages from {mid}..." in stderr,
+                bool(
+                    re.search(
+                        rf"Proceed to upgrade \S+ with {mid}\.\.\.",
+                        stderr,
+                    )
+                ),
                 # Remove messages.
                 bool(
                     re.search(
@@ -266,11 +273,7 @@ class CLISubCommandTests:
             pytest.param(("--manager", "apm") * 2, {"apm"}, id="duplicate_selectors"),
             pytest.param(("--apm",) * 2, {"apm"}, id="duplicate_flag_selectors"),
             pytest.param(
-                (
-                    "--manager",
-                    "apm",
-                    "--apm",
-                ),
+                ("--manager", "apm", "--apm"),
                 {"apm"},
                 id="duplicate_mixed_selectors",
             ),
