@@ -17,6 +17,9 @@
 
 # pylint: disable=redefined-outer-name
 
+from __future__ import annotations
+
+import dataclasses
 import json
 
 import pytest
@@ -64,8 +67,8 @@ class TestInstalled(CLISubCommandTests, CLITableTests):
             for pkg in info["packages"]:
                 assert isinstance(pkg, dict)
 
-                fields = set(Package.__dataclass_fields__)
-                assert set(pkg) == fields
+                fields = {f.name for f in dataclasses.fields(Package)}
+                assert set(pkg).issubset(fields)
 
-                for f in fields:
+                for f in pkg:
                     assert isinstance(pkg[f], str) or pkg[f] is None
