@@ -15,7 +15,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import pytest
+from click_extra.tests.conftest import destructive
 
 from ..pool import pool
 from .test_cli import CLISubCommandTests
@@ -36,6 +39,7 @@ def subcmd(create_config):
 
 
 class TestRestore(CLISubCommandTests):
+    @destructive
     def test_default_all_managers(self, invoke, create_config):
         toml_path = create_config(
             "all-managers.toml",
@@ -55,6 +59,7 @@ class TestRestore(CLISubCommandTests):
         assert "all-managers.toml" in result.stderr
         self.check_manager_selection(result)
 
+    @destructive
     @pytest.mark.parametrize("mid", pool.default_manager_ids)
     def test_single_manager(self, invoke, create_config, mid):
         toml_path = create_config(
@@ -88,6 +93,7 @@ class TestRestore(CLISubCommandTests):
         assert "unrecognized.toml" in result.stderr
         assert "Ignore [random_section] section" in result.stderr
 
+    @destructive
     def test_restore_single_manager(self, invoke, create_config):
         toml_path = create_config(
             "pip-npm-dummy.toml",
@@ -106,6 +112,7 @@ class TestRestore(CLISubCommandTests):
         assert "Restore pip" not in result.stderr
         assert "Restore npm" in result.stderr
 
+    @destructive
     def test_restore_excluded_manager(self, invoke, create_config):
         toml_path = create_config(
             "pip-npm-dummy.toml",
